@@ -9,7 +9,6 @@ Replaces relevant parts of the Dataflow package. Most of the functions have been
 
 <https://github.com/tensorpack/dataflow/blob/master/dataflow/dataflow/parallel.py>
 <https://github.com/tensorpack/dataflow/blob/master/dataflow/dataflow/parallel_map.py>
-
 """
 
 import atexit
@@ -23,13 +22,14 @@ import uuid
 import weakref
 from abc import ABC, abstractmethod
 from contextlib import contextmanager
-from typing import Any, Callable, Iterator, List, no_type_check
+from typing import Any, Callable, Iterator, no_type_check
 
 import zmq
 
 from ..utils.concurrency import StoppableThread, enable_death_signal, start_proc_mask_signal
-from ..utils.logger import logger
-from .base import DataFlow, DataFlowReentrantGuard, DataFlowTerminated, ProxyDataFlow
+from ..utils.error import DataFlowTerminatedError
+from ..utils.logger import LoggingRecord, logger
+from .base import DataFlow, DataFlowReentrantGuard, ProxyDataFlow
 from .common import RepeatedData
 from .serialize import PickleSerializer
 
@@ -39,7 +39,7 @@ def del_weakref(x):
     """delete weakref"""
     instance = x()
     if instance is not None:
-        instance.__del__()
+        del instance
 
 
 @no_type_check
@@ -48,15 +48,15 @@ def _zmq_catch_error(name):
     try:
         yield
     except zmq.ContextTerminated as exc:
-        logger.info("[%s] Context terminated.", name)
-        raise DataFlowTerminated() from exc
+        logger.info(LoggingRecord(f"_zmq_catch_error: [{name}] Context terminated."))
+        raise DataFlowTerminatedError() from exc
     except zmq.ZMQError as exc:
         if exc.errno == errno.ENOTSOCK:  # socket closed
-            logger.info("[%s] Socket closed.", name)
-            raise DataFlowTerminated() from exc
-        raise ValueError from exc
+            logger.info(LoggingRecord(f"_zmq_catch_error: [{name}]  Socket closed."))
+            raise DataFlowTerminatedError() from exc
+        raise ValueError() from exc
     except Exception as exc:
-        raise ValueError from exc
+        raise ValueError() from exc
 
 
 @no_type_check
@@ -78,8 +78,8 @@ def _get_pipe_name(name):
 class _ParallelMapData(ProxyDataFlow, ABC):
     def __init__(self, df: DataFlow, buffer_size: int, strict: bool = False) -> None:
         super().__init__(df)
-        if not buffer_size:
-            raise ValueError("buffer_size must be a positive number")
+        if buffer_size <= 0:
+            raise ValueError(f"buffer_size must be a positive number, got {buffer_size}")
         self._buffer_size = buffer_size
         self._buffer_occupancy = 0  # actual #elements in buffer, only useful in strict mode
         self._strict = strict
@@ -90,17 +90,17 @@ class _ParallelMapData(ProxyDataFlow, ABC):
             df = RepeatedData(self.df, -1)
         else:
             df = self.df  # type: ignore
-        self._iter = df.__iter__()
+        self._iter = iter(df)
 
     @no_type_check
     @abstractmethod
     def _recv(self):
-        raise NotImplementedError
+        raise NotImplementedError()
 
     @no_type_check
     @abstractmethod
     def _send(self, dp: Any):
-        raise NotImplementedError
+        raise NotImplementedError()
 
     @no_type_check
     def _recv_filter_none(self):
@@ -137,7 +137,7 @@ class _ParallelMapData(ProxyDataFlow, ABC):
         for dp in self._iter:
             self._send(dp)
             yield self._recv_filter_none()
-        self._iter = self.df.__iter__()  # refresh
+        self._iter = iter(self.df)  # refresh
 
         # first clear the buffer, then fill
         for k in range(self._buffer_size):
@@ -162,18 +162,19 @@ class MultiThreadMapData(_ParallelMapData):
     The semantics of this class is **identical** to `MapData` except for the ordering.
     Threads run in parallel and can take different time to run the
     mapping function. Therefore, the order of datapoints won't be preserved.
-    When ``strict=True``, ``MultiThreadMapData(df, ...)``
-    is guaranteed to produce the exact set of data as ``MapData(df, ...)``,
-    if both are iterated until ``StopIteration``. But the produced data will have different ordering.
-    The behavior of strict mode is undefined if the given dataflow ``df`` is infinite.
-    When ``strict=False``, the data that's produced by ``MultiThreadMapData(df, ...)``
-    is a reordering of the data produced by ``RepeatedData(MapData(df, ...), -1)``.
-    In other words, first pass of ``MultiThreadMapData.__iter__`` may contain
-    datapoints from the second pass of ``df.__iter__``.
+    When `strict=True`, `MultiThreadMapData(df, ...)`
+    is guaranteed to produce the exact set of data as `MapData(df, ...)`,
+    if both are iterated until `StopIteration`. But the produced data will have different ordering.
+    The behavior of strict mode is undefined if the given dataflow `df` is infinite.
+    When `strict=False`, the data that's produced by `MultiThreadMapData(df, ...)`
+    is a re-ordering of the data produced by `RepeatedData(MapData(df, ...), -1)`.
+    In other words, first pass of `MultiThreadMapData.__iter__` may contain
+    datapoints from the second pass of `df.__iter__`.
+
     Note:
         1. You should avoid starting many threads in your main process to reduce GIL contention.
            The threads will only start in the process which calls `reset_state()`.
-           Therefore you can use ``MultiProcessRunnerZMQ(MultiThreadMapData(...), 1)``
+           Therefore you can use `MultiProcessRunnerZMQ(MultiThreadMapData(...), 1)`
            to reduce GIL contention.
     """
 
@@ -214,12 +215,13 @@ class MultiThreadMapData(_ParallelMapData):
         strict: bool = False,
     ):
         """
-        :param df: the dataflow to map
-        :param num_thread: number of threads to use
-        :param map_func: datapoint -> datapoint | None. Return None to
-                         discard/skip the datapoint.
-        :param buffer_size: number of datapoints in the buffer
-        :param strict: use "strict mode", see notes above.
+        Args:
+            df: the dataflow to map
+            num_thread: number of threads to use
+            map_func: datapoint -> datapoint | None. Return None to
+                      discard/skip the datapoint.
+            buffer_size: number of datapoints in the buffer
+            strict: use "strict mode", see notes above.
         """
         if strict:
             # In strict mode, buffer size cannot be larger than the total number of datapoints
@@ -235,7 +237,7 @@ class MultiThreadMapData(_ParallelMapData):
         self._strict = strict
         self.num_thread = num_thread
         self.map_func = map_func
-        self._threads: List[Any] = []
+        self._threads: list[Any] = []
         self._evt = None
 
     def reset_state(self) -> None:
@@ -283,13 +285,13 @@ class _MultiProcessZMQDataFlow(DataFlow, ABC):
         if os.name == "nt":
             raise EnvironmentError("ZMQ IPC doesn't support windows")
         self._reset_done = False
-        self._procs: List[Any] = []
+        self._procs: list[Any] = []
         self.context = None
         self.socket = None
 
     def reset_state(self) -> Any:
         """
-        All forked dataflows should only be reset **once and only once** in spawned processes.
+        All forked dataflows should only be reset once and only once in spawned processes.
         Subclasses should call this method with super.
         """
         assert not self._reset_done, "reset_state() was called twice! This violates the API of DataFlow!"
@@ -312,7 +314,8 @@ class _MultiProcessZMQDataFlow(DataFlow, ABC):
             for x in self._procs:
                 x.terminate()
                 x.join(5)
-            logger.info("%s successfully cleaned-up.", type(self).__name__)
+            logger.info(LoggingRecord(f"_MultiProcessZMQDataFlow [{type(self).__name__}] successfully cleaned-up."))
+
         except Exception:  # pylint: disable=W0703
             pass
 
@@ -323,9 +326,12 @@ def _bind_guard(sock, name):
         sock.bind(name)
     except zmq.ZMQError:
         logger.error(
-            "ZMQError in socket.bind('{name}'). Perhaps you're using pipes on a non-local file system. "
-            "See documentation of MultiProcessRunnerZMQ for more information."
+            LoggingRecord(
+                f"ZMQError in socket.bind('{name}'). Perhaps you're using pipes on a non-local file system. "
+                "See documentation of MultiProcessRunnerZMQ for more information."
+            )
         )
+
         raise
 
 
@@ -333,17 +339,17 @@ class MultiProcessMapData(_ParallelMapData, _MultiProcessZMQDataFlow):
     """
     Same as `MapData`, but start processes to run the mapping function,
     and communicate with ZeroMQ pipe.
-    The semantics of this class is **identical** to `MapData` except for the ordering.
+    The semantics of this class is identical to `MapData` except for the ordering.
     Processes run in parallel and can take different time to run the
     mapping function. Therefore, the order of datapoints won't be preserved.
-    When ``strict=True``, ``MultiProcessMapData(df, ...)``
-    is guaranteed to produce the exact set of data as ``MapData(df, ...)``,
-    if both are iterated until ``StopIteration``. But the produced data will have different ordering.
-    The behavior of strict mode is undefined if the given dataflow ``df`` is infinite.
-    When ``strict=False``, the data that's produced by ``MultiProcessMapData(df, ...)``
-    is a reordering of the data produced by ``RepeatedData(MapData(df, ...), -1)``.
-    In other words, first pass of ``MultiProcessMapData.__iter__`` may contain
-    datapoints from the second pass of ``df.__iter__``.
+    When `strict=True`, `MultiProcessMapData(df, ...)`
+    is guaranteed to produce the exact set of data as `MapData(df, ...)`,
+    if both are iterated until `StopIteration`. But the produced data will have different ordering.
+    The behavior of strict mode is undefined if the given dataflow `df` is infinite.
+    When `strict=False`, the data that's produced by `MultiProcessMapData(df, ...)`
+    is a reordering of the data produced by `RepeatedData(MapData(df, ...), -1)`.
+    In other words, first pass of `MultiProcessMapData.__iter__` may contain
+    datapoints from the second pass of `df.__iter__`.
     """
 
     class _Worker(mp.Process):
@@ -358,7 +364,7 @@ class MultiProcessMapData(_ParallelMapData, _MultiProcessZMQDataFlow):
         @no_type_check
         def run(self):
             enable_death_signal(_warn=self.identity == b"0")
-            ctx = zmq.Context()  # pylint: disable=E0110
+            ctx = zmq.Context()
             socket = ctx.socket(zmq.REP)
             socket.setsockopt(zmq.IDENTITY, self.identity)
             socket.set_hwm(self.hwm)
@@ -379,11 +385,12 @@ class MultiProcessMapData(_ParallelMapData, _MultiProcessZMQDataFlow):
         strict: bool = False,
     ) -> None:
         """
-        :param df: the dataflow to map
-        :param num_proc: number of threads to use
-        :param map_func: datapoint -> datapoint | None. Return None to
-        :param buffer_size: number of datapoints in the buffer
-        :param strict: use "strict mode", see notes above.
+        Args:
+            df: the dataflow to map
+            num_proc: number of threads to use
+            map_func: datapoint -> datapoint | None. Return None to
+            buffer_size: number of datapoints in the buffer
+            strict: use "strict mode", see notes above.
         """
         if strict:
             # In strict mode, buffer size cannot be larger than the total number of datapoints
@@ -394,8 +401,8 @@ class MultiProcessMapData(_ParallelMapData, _MultiProcessZMQDataFlow):
 
         _ParallelMapData.__init__(self, df, buffer_size, strict)
         _MultiProcessZMQDataFlow.__init__(self)
-        if not num_proc:
-            raise ValueError("num_proc must be a positive number")
+        if num_proc <= 0:
+            raise ValueError(f"num_proc must be a positive number, got {num_proc}")
         self.num_proc = num_proc
         self.map_func = map_func
         self._strict = strict
@@ -410,7 +417,7 @@ class MultiProcessMapData(_ParallelMapData, _MultiProcessZMQDataFlow):
         _ParallelMapData.reset_state(self)
         self._guard = DataFlowReentrantGuard()
 
-        self.context = zmq.Context()  # type: ignore  # pylint: disable=E0110
+        self.context = zmq.Context()  # type: ignore
         self.socket = self.context.socket(zmq.DEALER)  # type: ignore
         self.socket.set_hwm(self._buffer_size * 2)  # type: ignore
         pipename = _get_pipe_name("dataflow-map")
