@@ -12,20 +12,22 @@ Some DataFlow classes for serialization. Many classes have been taken from
 
 import pickle
 from copy import copy
-from typing import Any, Iterable, Iterator, List, Optional, Tuple, Union
+from typing import Any, Iterable, Iterator, Optional, Union
 
 import numpy as np
 
-from .base import DataFlow, DataFlowResetStateNotCalled, RNGDataFlow
+from ..utils.error import DataFlowResetStateNotCalledError
+from .base import DataFlow, RNGDataFlow
 
 
 class DataFromList(RNGDataFlow):
     """Wrap a list of datapoints to a DataFlow"""
 
-    def __init__(self, lst: List[Any], shuffle: bool = True) -> None:
+    def __init__(self, lst: list[Any], shuffle: bool = True) -> None:
         """
-        :param lst: input list. Each element is a datapoint.
-        :param shuffle: shuffle data.
+        Args:
+            lst: input list. Each element is a datapoint.
+            shuffle: shuffle data.
         """
         super().__init__()
         self.lst = lst
@@ -44,7 +46,7 @@ class DataFromList(RNGDataFlow):
                 for k in idxs:
                     yield self.lst[k]
             else:
-                raise DataFlowResetStateNotCalled()
+                raise DataFlowResetStateNotCalledError()
 
 
 class DataFromIterable(DataFlow):
@@ -52,7 +54,8 @@ class DataFromIterable(DataFlow):
 
     def __init__(self, iterable: Iterable[Any]) -> None:
         """
-        :param iterable: an iterable object
+        Args:
+            iterable: an iterable object
         """
         self._itr = iterable
         self._len: Optional[int] = None
@@ -63,7 +66,7 @@ class DataFromIterable(DataFlow):
 
     def __len__(self) -> int:
         if self._len is None:
-            raise NotImplementedError
+            raise NotImplementedError()
         return self._len
 
     def __iter__(self) -> Iterator[Any]:
@@ -78,19 +81,20 @@ class FakeData(RNGDataFlow):
 
     def __init__(
         self,
-        shapes: List[Union[List[Any], Tuple[Any]]],
+        shapes: list[Union[list[Any], tuple[Any]]],
         size: int = 1000,
         random: bool = True,
         dtype: str = "float32",
-        domain: Tuple[Union[float, int], Union[float, int]] = (0, 1),
+        domain: tuple[Union[float, int], Union[float, int]] = (0, 1),
     ):
         """
-        :param  shapes: a list of lists/tuples. Shapes of each component.
-        :param  size: size of this DataFlow.
-        :param  random: whether to randomly generate data every iteration.
+        Args:
+            shapes: a list of lists/tuples. Shapes of each component.
+            size: size of this DataFlow.
+            random: whether to randomly generate data every iteration.
                         Note that merely generating the data could sometimes be time-consuming!
-        :param dtype: data type as string, or a list of data types.
-        :param domain: (min, max) tuple, or a list of such tuples
+            dtype: data type as string, or a list of data types.
+            domain: (min, max) tuple, or a list of such tuples
         """
 
         super().__init__()
@@ -107,7 +111,7 @@ class FakeData(RNGDataFlow):
 
     def __iter__(self) -> Iterator[Any]:
         if self.rng is None:
-            raise DataFlowResetStateNotCalled()
+            raise DataFlowResetStateNotCalledError()
         if self.random:
             for _ in range(self._size):
                 val = []
@@ -135,13 +139,15 @@ class PickleSerializer:
     @staticmethod
     def dumps(obj: Any) -> bytes:
         """
-        :param obj: bytes
+        Args:
+            obj: bytes
         """
         return pickle.dumps(obj, protocol=-1)
 
     @staticmethod
     def loads(buf: Any) -> Any:
         """
-        :param buf: bytes
+        Args:
+            buf: bytes
         """
         return pickle.loads(buf)
