@@ -18,12 +18,13 @@
 """
 Dataflows for calculating statistical values of the underlying dataset
 """
-from typing import Any, Optional, Tuple, Union
+
+from typing import Any, Optional, Union
 
 import numpy as np
 import numpy.typing as npt
 
-from ..utils.logger import logger
+from ..utils.logger import LoggingRecord, logger
 from ..utils.tqdm import get_tqdm
 from .base import DataFlow, ProxyDataFlow
 
@@ -32,34 +33,42 @@ class MeanFromDataFlow(ProxyDataFlow):
     """
     Get the mean of some dataflow. Takes a component from a dataflow and calculates iteratively the mean.
 
-    **Example:**
+    Example:
+        ```python
+        df: some dataflow
+        MeanFromDataFlow(df).start() # If you want to put MeanFromDataFlow at the end of a dataflow
 
-            df: some dataflow
-            MeanFromDataFlow(df).start() if you want to put MeanFromDataFlow at the end of a dataflow
-            df: some dataflow
-            df = MeanFromDataFlow(df)
+        or
+
+        df: some dataflow
+        df = MeanFromDataFlow(df)
 
         is also possible. Testing with the progress bar will stop once the requested size has been reached.
+        ```
+
+
     """
 
     def __init__(
         self,
         df: DataFlow,
-        axis: Optional[Union[int, Tuple[int], Tuple[int, int], Tuple[int, int, int]]] = None,
+        axis: Optional[Union[int, tuple[int], tuple[int, int], tuple[int, int, int]]] = None,
         key: Optional[str] = None,
         max_datapoints: Optional[int] = None,
     ):
         """
-        :param df: the dataflow to test
-        :param axis: The axis along which to calculate the mean. It will always calculate the mean along
-                     the dataflow length axis, which is the 0th axis. E.g. for calculating the mean of an image dataset
-                     use
+        Args:
+            df: the dataflow to test
+            axis: The axis along which to calculate the mean. It will always calculate the mean along
+                  the dataflow length axis, which is the 0th axis. E.g. for calculating the mean of an image dataset
+                  use
+                      ```python
+                      MeanFromDataFlow(df,key="image",axis=(0,1,2)).start()
+                      ```
 
-                         MeanFromDataFlow(df,key="image",axis=(0,1,2)).start()
-
-        :param key: The datapoint key giving the values to evaluate the mean. If None it tries to use
-                    the whole datapoint.
-        :param max_datapoints: Will stop considering datapoints with index>max_datapoints.
+            key: The datapoint key giving the values to evaluate the mean. If None it tries to use
+                 the whole datapoint.
+            max_datapoints: Will stop considering datapoints with index>max_datapoints.
         """
 
         super().__init__(df)
@@ -93,9 +102,9 @@ class MeanFromDataFlow(ProxyDataFlow):
 
         if not self._reset_called:
             self.df.reset_state()
-        itr = self.df.__iter__()
+        itr = iter(self.df)
 
-        logger.info("Calculating mean")
+        logger.info(LoggingRecord("Calculating mean"))
 
         len_df: Optional[int]
         try:
@@ -108,7 +117,6 @@ class MeanFromDataFlow(ProxyDataFlow):
         with get_tqdm(total=len_df) as status_bar:
             n = None
             for n, dp in enumerate(itr, 1):
-
                 if isinstance(dp, dict):
                     assert isinstance(self.key, str), self.key
                     val = dp[self.key]
@@ -135,12 +143,12 @@ class MeanFromDataFlow(ProxyDataFlow):
                     x = np.mean(val, axis=self.axis if inner_axis is None else inner_axis)
                     self.mean += (x - self.mean) / n
 
-                status_bar.update()  # type: ignore
+                status_bar.update()
                 if self.max_datapoints is not None:
                     if n == self.max_datapoints:
                         break
 
-        logger.info("Mean from %s datapoints along axis %s: %s", n, self.axis, self.mean)
+        logger.info(LoggingRecord(f"Mean from {n} datapoints along axis {self.axis}: {self.mean}"))
 
         return self.mean
 
@@ -150,37 +158,41 @@ class StdFromDataFlow(ProxyDataFlow):
     Gets the standard deviation of some dataflow. Takes a component from a dataflow and calculates iteratively
     the standard deviation.
 
-    **Example:**
-
-            df= ...
-            StdFromDataFlow(df).start()
+    Example:
+        ```python
+        df= ...
+        StdFromDataFlow(df).start()
 
         if you want to put  StdFromDataFlow at the end of a dataflow
 
-            df: some dataflow
-            df = StdFromDataFlow(df)
+        df: some dataflow
+        df = StdFromDataFlow(df)
 
         is also possible. The testing with the progress bar will stop once the requested size has been reached.
+        ```
+
     """
 
     def __init__(
         self,
         df: DataFlow,
-        axis: Optional[Union[int, Tuple[int], Tuple[int, int], Tuple[int, int, int]]] = None,
+        axis: Optional[Union[int, tuple[int], tuple[int, int], tuple[int, int, int]]] = None,
         key: Optional[str] = None,
         max_datapoints: Optional[int] = None,
     ):
         """
-        :param df: the dataflow to test
-        :param axis: The axis along which to calculate the mean. It will always calculate the std along
+        Args:
+            df: the dataflow to test
+            axis: The axis along which to calculate the mean. It will always calculate the std along
                      the dataflow length axis, which is the 0th axis. E.g. for calculating the mean of an image dataset
                      use
-
+                         ```python
                          StdFromDataFlow(df,key="image",axis=(0,1,2)).start()
+                         ```
 
-        :param key: The datapoint key giving the values to evaluate the std. If None it tries to use
-                    the whole datapoint.
-        :param max_datapoints: Will stop considering datapoints with index>max_datapoints.
+            key: The datapoint key giving the values to evaluate the std. If None it tries to use
+                 the whole datapoint.
+            max_datapoints: Will stop considering datapoints with index>max_datapoints.
         """
 
         super().__init__(df)
@@ -215,9 +227,9 @@ class StdFromDataFlow(ProxyDataFlow):
 
         if not self._reset_called:
             self.df.reset_state()
-        itr = self.df.__iter__()
+        itr = iter(self.df)
 
-        logger.info("Calculating standard deviation")
+        logger.info(LoggingRecord("Calculating standard deviation"))
         try:
             len_df = len(self.df)
         except NotImplementedError:
@@ -227,7 +239,6 @@ class StdFromDataFlow(ProxyDataFlow):
         n = None
         with get_tqdm(total=len_df) as status_bar:
             for n, dp in enumerate(itr, 1):
-
                 if isinstance(dp, dict):
                     assert isinstance(self.key, str), self.key
                     val = dp[self.key]
@@ -260,7 +271,7 @@ class StdFromDataFlow(ProxyDataFlow):
                     ex += x - k
                     ex2 += (x - k) * (x - k)
 
-                status_bar.update()  # type: ignore
+                status_bar.update()
                 if self.max_datapoints is not None:
                     if n == self.max_datapoints:
                         break
@@ -268,6 +279,6 @@ class StdFromDataFlow(ProxyDataFlow):
             var = (ex2 - (ex * ex) / n) / (n - 1)
             self.std = np.sqrt(var)
 
-        logger.info("Standard deviation from %s datapoints along axis %s: %s", n, self.axis, self.std)
+        logger.info(LoggingRecord(f"Standard deviation from {n} datapoints along axis {self.axis}: {self.std}"))
 
         return self.std
