@@ -16,23 +16,16 @@
 # limitations under the License.
 
 """
-Simple framework inspired by
+# Dataset concept: Building, training and evaluating datasets
 
-<https://huggingface.co/docs/datasets/>
+Simple framework inspired by <https://huggingface.co/docs/datasets/> for creating datasets.
 
-for creating datasets.
-
-Create an info card, a DataFlowBaseBuilder derived instance, possibly a category card and a
-DatasetBase derived instance to create a data set.
 """
 
-from ..utils.file_utils import pytorch_available
+from .adapter import *
 from .base import *
 from .dataflow_builder import DataFlowBaseBuilder
 from .info import *
 from .instances import *
 from .registry import *
 from .save import *
-
-if pytorch_available():
-    from .adapter import *
