@@ -16,25 +16,28 @@
 # limitations under the License.
 
 """
-Module for wrapping datasets into a pytorch dataset framework.
+Wrapping datasets into a PyTorch dataset framework
 """
 
 
 from typing import Any, Callable, Iterator, Mapping, Optional, Union
 
-from ..dataflow import CacheData, CustomDataFromList, MapData, RepeatedData
+from lazy_imports import try_import
+
+from ..dataflow import CustomDataFromList, MapData, RepeatedData
 from ..datapoint.image import Image
 from ..datasets.base import DatasetBase
 from ..mapper.maputils import LabelSummarizer
-from ..utils.detection_types import DP, JsonDict
-from ..utils.file_utils import pytorch_available
-from ..utils.logger import log_once, logger
+from ..utils.logger import LoggingRecord, log_once, logger
 from ..utils.settings import DatasetType, LayoutType, ObjectTypes, PageType, WordType
 from ..utils.tqdm import get_tqdm
+from ..utils.types import DP, JsonDict
 from .registry import get_dataset
 
-if pytorch_available():
+with try_import() as import_guard:
     from torch.utils.data import IterableDataset
+if not import_guard.is_successful():
+    from ..utils.mocks import IterableDataset  # type: ignore
 
 
 class DatasetAdapter(IterableDataset):  # type: ignore
@@ -43,9 +46,8 @@ class DatasetAdapter(IterableDataset):  # type: ignore
     pytorch frameworks (e.g. Detectron2). It wraps the dataset and defines the compulsory
     `__iter__` using  `dataflow.build` .
 
-    DatasetAdapter is meant for training and will therefore produce an infinite number of datapoints
+    `DatasetAdapter` is meant for training and will therefore produce an infinite number of datapoints
     by shuffling and restart iteration once the previous dataflow is exhausted.
-
     """
 
     def __init__(
@@ -54,18 +56,26 @@ class DatasetAdapter(IterableDataset):  # type: ignore
         cache_dataset: bool,
         image_to_framework_func: Optional[Callable[[DP], Optional[JsonDict]]] = None,
         use_token_tag: bool = True,
+        number_repetitions: int = -1,
         **build_kwargs: str,
     ) -> None:
         """
-        :param name_or_dataset: Registered name of the dataset or an instance.
-        :param cache_dataset: If set to true, it will cache the dataset (without loading images). If possible,
-                              some statistics, e.g. number of specific labels will be printed.
-        :param image_to_framework_func: A mapping function that converts image datapoints into the framework format
-        :param use_token_tag: Will only be used for dataset_type="token_classification". If use_token_tag=True, will use
-                              labels from sub category `WordType.token_tag` (with `B,I,O` suffix), otherwise
-                              `WordType.token_class`.
-        :param build_kwargs: optional parameters for defining the dataflow.
+        Args:
+            name_or_dataset: Registered name of the dataset or an instance.
+            cache_dataset: If set to `True`, it will cache the dataset (without loading images). If possible,
+                           some statistics, e.g. number of specific labels will be printed.
+            image_to_framework_func: A mapping function that converts image datapoints into the framework format
+            use_token_tag: Will only be used for dataset_type="token_classification". If `use_token_tag=True`, will use
+                           labels from sub category `WordType.token_tag` (with `B,I,O` suffix), otherwise
+                           `WordType.token_class`.
+            build_kwargs: optional parameters for defining the dataflow.
         """
+        if number_repetitions == -1 and not cache_dataset:
+            raise ValueError(
+                "Number of repetitions cannot be infinite when not caching the dataset. Instead try to"
+                " set a high number of repetitions"
+            )
+
         if isinstance(name_or_dataset, str):
             self.dataset = get_dataset(name_or_dataset)
         else:
@@ -74,27 +84,26 @@ class DatasetAdapter(IterableDataset):  # type: ignore
         df = self.dataset.dataflow.build(**build_kwargs)
 
         if cache_dataset:
-            logger.info("Yielding dataflow into memory and create torch dataset")
-            categories: Mapping[str, ObjectTypes] = {}
+            logger.info(LoggingRecord("Yielding dataflow into memory and create torch dataset"))
+            categories: Mapping[int, ObjectTypes] = {}
             _data_statistics = True
-            if self.dataset.dataset_info.type in (DatasetType.object_detection, DatasetType.sequence_classification):
+            if self.dataset.dataset_info.type in (DatasetType.OBJECT_DETECTION, DatasetType.SEQUENCE_CLASSIFICATION):
                 categories = self.dataset.dataflow.categories.get_categories(filtered=True)
-            elif self.dataset.dataset_info.type in (DatasetType.token_classification,):
+            elif self.dataset.dataset_info.type in (DatasetType.TOKEN_CLASSIFICATION,):
                 if use_token_tag:
                     categories = self.dataset.dataflow.categories.get_sub_categories(
-                        categories=LayoutType.word,
-                        sub_categories={LayoutType.word: [WordType.token_tag]},
+                        categories=LayoutType.WORD,
+                        sub_categories={LayoutType.WORD: [WordType.TOKEN_TAG]},
                         keys=False,
                         values_as_dict=True,
-                    )[LayoutType.word][WordType.token_tag]
+                    )[LayoutType.WORD][WordType.TOKEN_TAG]
                 else:
                     categories = self.dataset.dataflow.categories.get_sub_categories(
-                        categories=LayoutType.word, sub_categories={LayoutType.word: [WordType.token_class]}, keys=False
-                    )[LayoutType.word][WordType.token_class]
+                        categories=LayoutType.WORD, sub_categories={LayoutType.WORD: [WordType.TOKEN_CLASS]}, keys=False
+                    )[LayoutType.WORD][WordType.TOKEN_CLASS]
             else:
                 logger.info(
-                    "dataset is of type %s. Cannot generate statistics for this type of dataset",
-                    self.dataset.dataset_info.type,
+                    LoggingRecord(f"dataset is of type {self.dataset.dataset_info.type}. Cannot generate statistics.")
                 )
                 _data_statistics = False
 
@@ -119,37 +128,36 @@ class DatasetAdapter(IterableDataset):  # type: ignore
                             "images when needed and reduce memory costs!!!",
                             "warn",
                         )
-                    if self.dataset.dataset_info.type == DatasetType.object_detection:
+                    if self.dataset.dataset_info.type == DatasetType.OBJECT_DETECTION:
                         anns = dp.get_annotation()
-                        cat_ids = [int(ann.category_id) for ann in anns]
+                        cat_ids = [ann.category_id for ann in anns]
 
-                    elif self.dataset.dataset_info.type == DatasetType.sequence_classification:
-                        cat_ids = dp.summary.get_sub_category(PageType.document_type).category_id
+                    elif self.dataset.dataset_info.type == DatasetType.SEQUENCE_CLASSIFICATION:
+                        cat_ids = dp.summary.get_sub_category(PageType.DOCUMENT_TYPE).category_id
 
-                    elif self.dataset.dataset_info.type == DatasetType.token_classification:
-                        anns = dp.get_annotation()
+                    elif self.dataset.dataset_info.type == DatasetType.TOKEN_CLASSIFICATION:
+                        anns = dp.get_annotation(category_names=LayoutType.WORD)
                         if use_token_tag:
-                            cat_ids = [ann.get_sub_category(WordType.token_tag).category_id for ann in anns]
+                            cat_ids = [ann.get_sub_category(WordType.TOKEN_TAG).category_id for ann in anns]
                         else:
-                            cat_ids = [ann.get_sub_category(WordType.token_class).category_id for ann in anns]
+                            cat_ids = [ann.get_sub_category(WordType.TOKEN_CLASS).category_id for ann in anns]
 
                     if _data_statistics:
                         summarizer.dump(cat_ids)
 
                     datapoints.append(dp)
-                    status_bar.update()  # type: ignore
+                    status_bar.update()
 
             if _data_statistics:
                 summarizer.print_summary_histogram()
             self.number_datapoints = len(datapoints)
+            if not self.number_datapoints:
+                raise ValueError("DatasetAdapter receives no datapoints. Please check your dataflow build config.")
 
             df = CustomDataFromList(datapoints, shuffle=True)
-            if not image_to_framework_func:
-                df = RepeatedData(df, -1)
-            else:
-                df_list = CacheData(df).get_cache()
-                df = CustomDataFromList(df_list, shuffle=True)
-                df = RepeatedData(df, -1)
+            df = RepeatedData(df, number_repetitions)
+        else:
+            df = RepeatedData(df, number_repetitions)
 
         if image_to_framework_func:
             df = MapData(df, image_to_framework_func)
@@ -166,4 +174,4 @@ class DatasetAdapter(IterableDataset):  # type: ignore
         return len(self.df)
 
     def __getitem__(self, item: Any) -> None:
-        raise NotImplementedError
+        raise NotImplementedError()
