@@ -16,87 +16,89 @@
 # limitations under the License.
 
 """
-Module for storing dataset info (e.g. general meta data or categories)
+General meta-data or categories
 """
 
 from copy import copy
 from dataclasses import dataclass, field
 from itertools import chain
-from typing import Any, Dict, List, Literal, Mapping, Optional, Sequence, Set, Union, overload
+from typing import Any, Literal, Mapping, Optional, Sequence, Union, no_type_check, overload
 
-from ..utils.settings import DefaultType, ObjectTypes, TypeOrStr, get_type
+from ..utils.settings import DatasetType, ObjectTypes, TypeOrStr, get_type
 from ..utils.utils import call_only_once
 
 __all__ = ["DatasetInfo", "DatasetCategories", "get_merged_categories"]
 
 
 @overload
-def _get_dict(l: Sequence[ObjectTypes], name_as_key: Literal[True], starts_with: int = ...) -> Dict[ObjectTypes, str]:
+def _get_dict(l: Sequence[ObjectTypes], name_as_key: Literal[True], starts_with: int = ...) -> dict[ObjectTypes, int]:
     ...
 
 
 @overload
-def _get_dict(l: Sequence[ObjectTypes], name_as_key: Literal[False], starts_with: int = ...) -> Dict[str, ObjectTypes]:
+def _get_dict(l: Sequence[ObjectTypes], name_as_key: Literal[False], starts_with: int = ...) -> dict[int, ObjectTypes]:
     ...
 
 
 @overload
 def _get_dict(
     l: Sequence[ObjectTypes], name_as_key: bool, starts_with: int = ...
-) -> Union[Dict[ObjectTypes, str], Dict[str, ObjectTypes]]:
+) -> Union[dict[ObjectTypes, int], dict[int, ObjectTypes]]:
     ...
 
 
 def _get_dict(
     l: Sequence[ObjectTypes], name_as_key: bool, starts_with: int = 1
-) -> Union[Dict[ObjectTypes, str], Dict[str, ObjectTypes]]:
+) -> Union[dict[ObjectTypes, int], dict[int, ObjectTypes]]:
     """
     Converts a list into a dict, where keys/values are the list indices.
 
-    :param l: A list of categories
-    :param name_as_key: Whether to return the dict with category names as key (True)
-    :param starts_with: index count start
-    :return: A dictionary of list indices/list elements.
+    Args:
+        l: A list of categories
+        name_as_key: Whether to return the dict with category names as key (`True`)
+        starts_with: index count start
+
+    Returns:
+        A dictionary of list indices/list elements.
     """
     if name_as_key:
-        return {v: str(k) for k, v in enumerate(l, starts_with)}
-    return {str(k): v for k, v in enumerate(l, starts_with)}
+        return {v: k for k, v in enumerate(l, starts_with)}
+    return dict(enumerate(l, starts_with))
 
 
 @dataclass
 class DatasetInfo:
     """
-    DatasetInfo is a simple dataclass that stores some meta-data information about a dataset.
+    `DatasetInfo` is a simple dataclass that stores some meta-data information about a dataset.
 
-    `name`: Name of the dataset. Using the name you can retrieve the dataset from the
-    `registry.DatasetRegistry`.
-
-    `description`: Short description of the dataset.
-
-    `license`: License to the dataset.
-
-    `url`: url, where the dataset can be downloaded from.
-
-    `splits`: A dict of splits. The value must store the relative path, where the split can be found.
-
-    `type`: The type describes whether this is a dataset for object detection (pass 'OBJECT_DETECTION'),
-    sequence classification (pass 'SEQUENCE_CLASSIFICATION') or token classification ('TOKEN_CLASSIFICATION').
-    Optionally, pass `None`.
+     Attributes:
+        name: Name of the dataset. Using the name you can retrieve the dataset from the `registry.DatasetRegistry`.
+        description: Short description of the dataset.
+        license: License to the dataset.
+        url: url, where the dataset can be downloaded from.
+        splits: A `dict` of splits. The value must store the relative path, where the split can be found.
+        type: The type describes whether this is a dataset for object detection (pass 'OBJECT_DETECTION'),
+              sequence classification (pass 'SEQUENCE_CLASSIFICATION') or token classification ('TOKEN_CLASSIFICATION').
+              Optionally, pass `None`.
     """
 
     name: str
+    short_description: str = field(default="")
     description: str = field(default="")
     license: str = field(default="")
     url: Union[str, Sequence[str]] = field(default="")
     splits: Mapping[str, str] = field(default_factory=dict)
-    type: ObjectTypes = field(default=DefaultType.default_type)
+    type: DatasetType = field(default=DatasetType.DEFAULT)
 
     def get_split(self, key: str) -> str:
         """
         Get the split directory by its key (if it exists).
 
-        :param key: The key to a split (i.e. "train", "val", "test")
-        :return: The local directory path to the split. An empty string if the key doesn't exist.
+        Args:
+            key: The key to a split (i.e. `train`, `val`,`test`)
+
+        Returns:
+            The local directory path to the split. An empty string if the key doesn't exist.
         """
 
         return self.splits[key]
@@ -111,22 +113,26 @@ class DatasetCategories:
     for the index/category name relationship and guarantees that a sequence of natural numbers for the categories
     is always returned as the category-id even after replacing and/or filtering.
 
-    `init_categories`: A list of category names. The list must include all categories that can occur within the
-    annotations.
+    Attributes:
+        init_categories: A list of `category_name`s. The list must include all categories that can occur within the
+                         annotations.
+        init_sub_categories: A dict of categories/sub-categories. Each sub-category that can appear in the
+                             annotations in combination with a category must be listed.
 
-    `init_sub_categories`: A dict of categories/sub-categories. Each sub-category that can appear in the
-    annotations in combination with a category must be listed.
-
-    **Example:**
+    Example:
 
         An annotation file hast the category/sub-category combinations for three datapoints:
 
-            (cat1,s1),(cat1,s2), (cat2,s2).
+        ```python
+        (cat1,s1),(cat1,s2), (cat2,s2).
+        ```
 
         You must list `init_categories`, `init_sub_categories` as follows:
 
-            init_categories = [cat1,cat2]
-            init_sub_categories = {cat1: [s1,s2],cat2: [s2]}
+        ```python
+        init_categories = [cat1,cat2]
+        init_sub_categories = {cat1: [s1,s2],cat2: [s2]}
+        ```
 
     Use `filter_categories` or `set_cat_to_sub_cat` to filter or swap categories with sub-categories.
     """
@@ -143,13 +149,13 @@ class DatasetCategories:
     @overload
     def get_categories(
         self, *, name_as_key: Literal[True], init: bool = ..., filtered: bool = ...
-    ) -> Mapping[ObjectTypes, str]:
+    ) -> Mapping[ObjectTypes, int]:
         ...
 
     @overload
     def get_categories(
         self, *, name_as_key: Literal[False] = ..., init: bool = ..., filtered: bool = ...
-    ) -> Mapping[str, ObjectTypes]:
+    ) -> Mapping[int, ObjectTypes]:
         ...
 
     @overload
@@ -161,25 +167,28 @@ class DatasetCategories:
     @overload
     def get_categories(
         self, as_dict: Literal[True] = ..., name_as_key: bool = False, init: bool = False, filtered: bool = False
-    ) -> Union[Mapping[ObjectTypes, str], Mapping[str, ObjectTypes]]:
+    ) -> Union[Mapping[ObjectTypes, int], Mapping[int, ObjectTypes]]:
         ...
 
     def get_categories(
         self, as_dict: bool = True, name_as_key: bool = False, init: bool = False, filtered: bool = False
-    ) -> Union[Sequence[ObjectTypes], Mapping[ObjectTypes, str], Mapping[str, ObjectTypes]]:
+    ) -> Union[Sequence[ObjectTypes], Mapping[ObjectTypes, int], Mapping[int, ObjectTypes]]:
         """
         Get categories of a dataset. The returned value also respects modifications of the inventory like filtered
         categories of replaced categories with sub categories. However, you must correctly pass arguments to return the
         state you want.
 
-        :param as_dict: Will pass a dict if set to 'True' otherwise a list.
-        :param name_as_key: Categories are stored as key/value pair in a dict with integers as keys. name_as_key set to
-                            "False" will swap keys and values.
-        :param init: If set to "True" it will return the list/dict of categories as initially provided. Manipulations
-                     due to replacing/filtering will not be regarded.
-        :param filtered: If set to "True" will return an unfiltered list of all categories. If a replacing has been
-                         invoked selected sub categories will be returned.
-        :return: A dict of index/category names (or the other way around) or a list of category names.
+        Args:
+            as_dict: Will pass a dict if set to 'True' otherwise a list.
+            name_as_key: Categories are stored as key/value pair in a dict with integers as keys. `name_as_key` set to
+                         `False` will swap keys and values.
+            init: If set to `True` it will return the list/dict of categories as initially provided. Manipulations
+                  due to replacing/filtering will not be regarded.
+            filtered: If set to `True` will return an unfiltered list of all categories. If a replacing has been
+                      invoked selected sub categories will be returned.
+
+        Returns:
+            A dict of index/category names (or the other way around) or a list of category names.
         """
         if init:
             if as_dict:
@@ -208,14 +217,17 @@ class DatasetCategories:
         """
         Returns a dict of list with a category name and their sub categories.
 
-        :param categories: A single category or list of category names
-        :param sub_categories: A mapping of categories to sub category keys on which the result should be filtered. Only
+        Args:
+            categories: A single category or list of category names
+            sub_categories: A mapping of categories to sub category keys on which the result should be filtered. Only
                                relevant, if `keys=False`
-        :param keys: Will only pass keys if set to `True`.
-        :param values_as_dict: Will generate a dict with indices and sub category value names if set to `True`.
-        :param name_as_key: sub category values are stored as key/value pair in a dict with integers as keys.
+            keys: Will only pass keys if set to `True`.
+            values_as_dict: Will generate a dict with indices and sub category value names if set to `True`.
+            name_as_key: sub category values are stored as key/value pair in a dict with integers as keys.
                             name_as_key set to `False` will swap keys and values.
-        :return: Dict with all selected categories.
+
+        Returns:
+            Dict with all selected categories.
         """
         _categories: Sequence[ObjectTypes]
         if isinstance(categories, (ObjectTypes, str)):
@@ -229,8 +241,8 @@ class DatasetCategories:
         if sub_categories is None:
             sub_categories = {}
 
-        sub_cat: Dict[ObjectTypes, Union[ObjectTypes, List[ObjectTypes]]] = {}
-        for cat in _categories:  # pylint: disable=R1702
+        sub_cat: dict[ObjectTypes, Union[ObjectTypes, list[ObjectTypes]]] = {}
+        for cat in _categories:
             assert cat in self.get_categories(  # pylint: disable=E1135
                 as_dict=False, filtered=True
             ), f"{cat} not in categories. Maybe it has been replaced with sub category"
@@ -254,9 +266,9 @@ class DatasetCategories:
             for category, value in sub_cat.items():
                 if category not in sub_categories:
                     continue
-                sub_cat_tmp: Dict[str, Union[Dict[str, str], Sequence[str]]] = {}
+                sub_cat_tmp: dict[str, Union[dict[int, ObjectTypes], dict[ObjectTypes, int], Sequence[str]]] = {}
                 sub_categories_list: Union[
-                    ObjectTypes, str, List[Sequence[Union[ObjectTypes, str]]], Sequence[Union[ObjectTypes, str]]
+                    ObjectTypes, str, list[Sequence[Union[ObjectTypes, str]]], Sequence[Union[ObjectTypes, str]]
                 ]
                 if isinstance(sub_categories[category], ObjectTypes):
                     sub_categories_list = [sub_categories[category]]
@@ -267,14 +279,12 @@ class DatasetCategories:
                         continue
                     if values_as_dict:
                         if not name_as_key:
-                            sub_cat_tmp[sub_cat_key] = {
-                                str(k): v
-                                for k, v in enumerate(self.init_sub_categories[category][get_type(sub_cat_key)], 1)
-                            }
+                            sub_cat_tmp[sub_cat_key] = dict(
+                                enumerate(self.init_sub_categories[category][get_type(sub_cat_key)], 1)
+                            )
                         else:
                             sub_cat_tmp[sub_cat_key] = {
-                                v: str(k)
-                                for k, v in enumerate(self.init_sub_categories[category][get_type(sub_cat_key)], 1)
+                                v: k for k, v in enumerate(self.init_sub_categories[category][get_type(sub_cat_key)], 1)
                             }
                     else:
                         sub_cat_tmp[sub_cat_key] = self.init_sub_categories[category][get_type(sub_cat_key)]
@@ -284,7 +294,7 @@ class DatasetCategories:
         return sub_cat
 
     @call_only_once
-    def set_cat_to_sub_cat(self, cat_to_sub_cat: Dict[TypeOrStr, TypeOrStr]) -> None:
+    def set_cat_to_sub_cat(self, cat_to_sub_cat: dict[TypeOrStr, TypeOrStr]) -> None:
         """
         Change category representation if sub-categories are available. Pass a dictionary of the main category
         and the requested sub-category. This will change the dictionary of categories and the category names
@@ -294,19 +304,21 @@ class DatasetCategories:
         This method can only be called once per object. Re-setting or further replacing of categories would make the
         code messy and is therefore not allowed.
 
-        **Example:**
-
-                  cat_to_sub_cat={cat1: sub_cat1}
+        Example:
+            ```python
+            cat_to_sub_cat={cat1: sub_cat1}
+            ```
 
             will replace cat1 with sub_cat1 as category. This will also be respected when returning datapoints.
 
-        :param cat_to_sub_cat: A dict of pairs of category/sub-category. Note that the combination must be available
-                               according to the initial settings.
+        Args:
+            cat_to_sub_cat: A dict of pairs of category/sub-category. Note that the combination must be available
+                            according to the initial settings.
         """
 
         _cat_to_sub_cat = {get_type(key): get_type(value) for key, value in cat_to_sub_cat.items()}
         if not self._allow_update:
-            raise PermissionError("Replacing categories with sub categories is not allowed")
+            raise RuntimeWarning("Replacing categories with sub categories is not allowed")
         self._categories_update = self.init_categories
         categories = self.get_categories(name_as_key=True)
         cats_or_sub_cats = [
@@ -323,16 +335,17 @@ class DatasetCategories:
             self._categories_update = _categories_update_list
 
     @call_only_once
-    def filter_categories(self, categories: Union[TypeOrStr, List[TypeOrStr]]) -> None:
+    def filter_categories(self, categories: Union[TypeOrStr, list[TypeOrStr]]) -> None:
         """
         Filter categories of a dataset. This will keep all the categories chosen and remove all others.
         This method can only be called once per object.
 
-        :param categories: A single category name or a list of category names.
+        Args:
+            categories: A single `category_name` or a list of `category_name`s.
         """
 
         if not self._allow_update:
-            raise PermissionError("Filtering categories is not allowed")
+            raise RuntimeWarning("Filtering categories is not allowed")
         if isinstance(categories, (ObjectTypes, str)):
             categories = [get_type(categories)]
         else:
@@ -345,13 +358,14 @@ class DatasetCategories:
     @property
     def cat_to_sub_cat(self) -> Optional[Mapping[ObjectTypes, ObjectTypes]]:
         """
-        cat_to_sub_cat
+        `cat_to_sub_cat`
         """
         return self._cat_to_sub_cat
 
     def is_cat_to_sub_cat(self) -> bool:
         """
-        returns `True` if a category is replaced with sub categories
+        Returns:
+            `True` if a category is replaced with sub categories
         """
         if self._cat_to_sub_cat is not None:
             return True
@@ -359,7 +373,8 @@ class DatasetCategories:
 
     def is_filtered(self) -> bool:
         """
-        return `True` if categories are filtered
+        Returns:
+            `True` if categories are filtered
         """
         if hasattr(self, "_categories_filter_update"):
             return True
@@ -370,6 +385,7 @@ class DatasetCategories:
         assert all(isinstance(val, dict) for val in self.init_sub_categories.values())
 
 
+@no_type_check
 def get_merged_categories(*categories: DatasetCategories) -> DatasetCategories:
     """
     Given a set of `DatasetCategories`, a `DatasetCategories` instance will be returned that summarize the category
@@ -379,8 +395,11 @@ def get_merged_categories(*categories: DatasetCategories) -> DatasetCategories:
     as well but no sub category than the merged dataset will have no sub categories at all. Whereas in a similar setting
     dataset B has sub category `foo`:`bak`, then `bak` will be an optional sub category for the merged dataset as well.
 
-    :param categories: A tuple/list of dataset categories
-    :return: An instance of `DatasetCategories` to be used as `DatasetCategories` for merged datasets
+    Args:
+        categories: A tuple/list of dataset categories
+
+    Returns:
+        An instance of `DatasetCategories` to be used as `DatasetCategories` for merged datasets
     """
 
     # working with lists is not possible as the order of categories is important here
@@ -414,14 +433,32 @@ def get_merged_categories(*categories: DatasetCategories) -> DatasetCategories:
         # form a set of possible sub category values. To get a list of all values from all dataset, take the union
         intersect_init_sub_cat_values = {}
         for sub_cat_key in intersect_sub_cat_per_key:
-            val: Set[ObjectTypes] = set()
+            val: set[ObjectTypes] = set()
             for cat in categories:
                 val.update(cat.init_sub_categories[key][sub_cat_key])
             intersect_init_sub_cat_values[sub_cat_key] = list(val)
         intersect_init_sub_cat[key] = intersect_init_sub_cat_values
 
+    # Building sub cats such that the result is deterministic. Because we use sets in several occasions above the
+    # construction is not deterministic but guarantees for unique values in all sub categories. Now we build the
+    # ensemble dict of sub categories where we guarantee unique values on one hand side and always maintain the
+    # same arrangements for all category/ sub category lists
+    init_sub_cat: dict[ObjectTypes, Any] = {}
+    for category in categories:
+        for cat in intersect_sub_cat_keys:
+            for sub_cat_key in category.init_sub_categories[cat]:
+                if sub_cat_key in intersect_init_sub_cat[cat]:
+                    for sub_cat_val in category.init_sub_categories[cat][sub_cat_key]:
+                        if sub_cat_val in intersect_init_sub_cat[cat][sub_cat_key]:
+                            if cat not in init_sub_cat:
+                                init_sub_cat[cat] = {}
+                            if sub_cat_key not in init_sub_cat[cat]:
+                                init_sub_cat[cat][sub_cat_key] = [sub_cat_val]
+                            else:
+                                init_sub_cat[cat][sub_cat_key].append(sub_cat_val)
+
     # Next, build the DatasetCategories instance.
-    merged_categories = DatasetCategories(init_categories=init_categories, init_sub_categories=intersect_init_sub_cat)
+    merged_categories = DatasetCategories(init_categories=init_categories, init_sub_categories=init_sub_cat)
     merged_categories._categories_update = categories_update  # pylint: disable = W0212
     merged_categories._allow_update = False  # pylint: disable = W0212
     setattr(merged_categories, "_categories_filter_update", categories_filtered)
