@@ -30,9 +30,10 @@ Module for Fintabnet dataset. Place the dataset as follows
     ├── FinTabNet_1.0.0_table_train.jsonl
     ├── FinTabNet_1.0.0_table_val.jsonl
 """
+from __future__ import annotations
 
 from pathlib import Path
-from typing import List, Mapping, Sequence, Union
+from typing import Mapping, Sequence, Union
 
 from ...dataflow import DataFlow, MapData, MultiProcessMapData
 from ...dataflow.common import FlattenData
@@ -42,10 +43,10 @@ from ...mapper.cats import cat_to_sub_cat, filter_cat
 from ...mapper.maputils import curry
 from ...mapper.misc import image_ann_to_image, maybe_ann_to_sub_image
 from ...mapper.pubstruct import pub_to_image
-from ...utils.detection_types import JsonDict
 from ...utils.file_utils import set_mp_spawn
-from ...utils.logger import logger
+from ...utils.logger import LoggingRecord, logger
 from ...utils.settings import CellType, DatasetType, LayoutType, ObjectTypes, TableType
+from ...utils.types import PubtabnetDict
 from ...utils.utils import to_bool
 from ..base import _BuiltInDataset
 from ..dataflow_builder import DataFlowBaseBuilder
@@ -53,6 +54,7 @@ from ..info import DatasetCategories, DatasetInfo
 from ..registry import dataset_registry
 
 _NAME = "fintabnet"
+_SHORT_DESCRIPTION = "FinTabNet dataset contains complex tables from the annual reports of S&P 500 companies."
 _DESCRIPTION = (
     "FinTabNet dataset contains complex tables from the annual reports of S&P 500 companies with detailed \n"
     " table structure annotations to help train and test structure recognition. \n"
@@ -82,38 +84,38 @@ _URL = (
     "fintabnet.tar.gz?_ga=2.17492593.994196051.1634564576-1173244232.1625045842"
 )
 _SPLITS: Mapping[str, str] = {"train": "train", "val": "val", "test": "test"}
-_TYPE = DatasetType.object_detection
+_TYPE = DatasetType.OBJECT_DETECTION
 _LOCATION = "fintabnet"
 _ANNOTATION_FILES: Mapping[str, str] = {
     "train": "FinTabNet_1.0.0_table_train.jsonl",
     "test": "FinTabNet_1.0.0_table_test.jsonl",
     "val": "FinTabNet_1.0.0_table_val.jsonl",
 }
-_INIT_CATEGORIES = [LayoutType.table, LayoutType.cell, TableType.item]
+_INIT_CATEGORIES = [LayoutType.TABLE, LayoutType.CELL, TableType.ITEM]
 _SUB_CATEGORIES: Mapping[ObjectTypes, Mapping[ObjectTypes, Sequence[ObjectTypes]]]
 _SUB_CATEGORIES = {
-    LayoutType.cell: {
-        CellType.header: [CellType.header, CellType.body],
-        CellType.row_number: [],
-        CellType.column_number: [],
-        CellType.row_span: [],
-        CellType.column_span: [],
-        CellType.spanning: [CellType.spanning, LayoutType.cell],
+    LayoutType.CELL: {
+        CellType.HEADER: [CellType.HEADER, CellType.BODY],
+        CellType.ROW_NUMBER: [],
+        CellType.COLUMN_NUMBER: [],
+        CellType.ROW_SPAN: [],
+        CellType.COLUMN_SPAN: [],
+        CellType.SPANNING: [CellType.SPANNING, LayoutType.CELL],
     },
-    TableType.item: {TableType.item: [LayoutType.row, LayoutType.column]},
-    CellType.header: {
-        CellType.row_number: [],
-        CellType.column_number: [],
-        CellType.row_span: [],
-        CellType.column_span: [],
-        CellType.spanning: [CellType.spanning, LayoutType.cell],
+    TableType.ITEM: {TableType.ITEM: [LayoutType.ROW, LayoutType.COLUMN]},
+    CellType.HEADER: {
+        CellType.ROW_NUMBER: [],
+        CellType.COLUMN_NUMBER: [],
+        CellType.ROW_SPAN: [],
+        CellType.COLUMN_SPAN: [],
+        CellType.SPANNING: [CellType.SPANNING, LayoutType.CELL],
     },
-    CellType.body: {
-        CellType.row_number: [],
-        CellType.column_number: [],
-        CellType.row_span: [],
-        CellType.column_span: [],
-        CellType.spanning: [CellType.spanning, LayoutType.cell],
+    CellType.BODY: {
+        CellType.ROW_NUMBER: [],
+        CellType.COLUMN_NUMBER: [],
+        CellType.ROW_SPAN: [],
+        CellType.COLUMN_SPAN: [],
+        CellType.SPANNING: [CellType.SPANNING, LayoutType.CELL],
     },
 }
 
@@ -128,12 +130,20 @@ class Fintabnet(_BuiltInDataset):
 
     @classmethod
     def _info(cls) -> DatasetInfo:
-        return DatasetInfo(name=_NAME, description=_DESCRIPTION, license=_LICENSE, url=_URL, splits=_SPLITS, type=_TYPE)
+        return DatasetInfo(
+            name=_NAME,
+            short_description=_SHORT_DESCRIPTION,
+            description=_DESCRIPTION,
+            license=_LICENSE,
+            url=_URL,
+            splits=_SPLITS,
+            type=_TYPE,
+        )
 
     def _categories(self) -> DatasetCategories:
         return DatasetCategories(init_categories=_INIT_CATEGORIES, init_sub_categories=_SUB_CATEGORIES)
 
-    def _builder(self) -> "FintabnetBuilder":
+    def _builder(self) -> FintabnetBuilder:
         return FintabnetBuilder(location=_LOCATION, annotation_files=_ANNOTATION_FILES)
 
 
@@ -147,28 +157,22 @@ class FintabnetBuilder(DataFlowBaseBuilder):
         Returns a dataflow from which you can stream datapoints of images. The following arguments affect the returns
         of the dataflow:
 
-        `split:` Split of the dataset. Can be `train`,`val` or `test`. Default: `val`
+        Args:
+            kwargs:
+                (split) Split of the dataset. Can be `train`, `val` or `test`. Default: `val`
+                (build_mode) Returns the full image or crops a table according to the table bounding box. Pass `table`
+                             if you only want the cropped table. Default: `""`
+                (max_datapoints) Will stop iterating after `max_datapoints`. Default: `None`
+                (rows_and_cols) Will add 'item' image annotations that either represent a row or a column of a table.
+                                Default: `True`
+                (load_image) Will load the image for each datapoint. Default: `False`
+                (use_multi_proc) Uses multiple processes for PDF conversion. Default: `True`
+                (use_multi_proc_strict) Uses strict mode in multiprocessing. Default: `False`
+                (fake_score) Adds a fake score so that annotations look like predictions. Default: `False`
+                (pubtables_like) Treats the dataset as PubTables-like. Default: `False`
 
-        `build_mode:` Returns the full image or crops a table according to the table bounding box. Pass `table`
-                           if you only want the cropped table. Default: ""
-
-        `max_datapoints:` Will stop iterating after max_datapoints. Default: `None`
-
-        `rows_and_cols:` Will add a 'item' image annotations that either represent a row or a column of a table.
-                              Note, that the type of the item (i.e. being a row or a column) can be inferred from the
-                              sub category added. Note further, that "item" are not originally part of the annotations
-                              and are inferred from cell positions and their associated table semantic. Default: `True`
-        `load_image:` Will load the image for each datapoint.  Default: `False`
-
-        `use_multi_proc:` As the original files are stored as pdf conversion into a numpy array is time-consuming.
-                        When setting use_multi_proc to True is will use several processes depending on the number
-                        of CPUs available.
-
-        `use_multi_proc_strict:` Will use strict mode in multiprocessing.
-
-        `fake_score:` Will add a fake score so that annotations look like predictions
-
-        :return: dataflow
+        Returns:
+            Dataflow
         """
 
         split = str(kwargs.get("split", "val"))
@@ -182,7 +186,7 @@ class FintabnetBuilder(DataFlowBaseBuilder):
         pubtables_like = kwargs.get("pubtables_like", False)
 
         if build_mode and not load_image:
-            logger.info("When 'build_mode' is set to True will reset 'load_image' to True")
+            logger.info(LoggingRecord("When 'build_mode' is set to True will reset 'load_image' to True"))
             load_image = True
 
         if use_multi_proc or use_multi_proc_strict:
@@ -191,7 +195,7 @@ class FintabnetBuilder(DataFlowBaseBuilder):
         if max_datapoints is not None:
             max_datapoints = int(max_datapoints)
         if kwargs.get("build_mode", "") != "table":
-            logger.info("Logic will display only only table per page, even if there are more!!")
+            logger.info(LoggingRecord("Logic will display only only table per page, even if there are more!!"))
 
         # Load
         df: DataFlow
@@ -200,18 +204,17 @@ class FintabnetBuilder(DataFlowBaseBuilder):
 
         # Map
         @curry
-        def _map_filename(dp: JsonDict, workdir: Path) -> JsonDict:
+        def _map_filename(dp: PubtabnetDict, workdir: Path) -> PubtabnetDict:
             dp["filename"] = workdir / "pdf" / dp["filename"]
             return dp
 
-        map_filename = _map_filename(self.get_workdir())  # pylint: disable=E1120  # 259
-        df = MapData(df, map_filename)
+        df = MapData(df, _map_filename(self.get_workdir()))
 
         buffer_size = 200 if max_datapoints is None else min(max_datapoints, 200) - 1
 
         pub_mapper = pub_to_image(
-            self.categories.get_categories(name_as_key=True, init=True),
-            load_image,
+            categories_name_as_key=self.categories.get_categories(name_as_key=True, init=True),
+            load_image=load_image,
             fake_score=fake_score,
             rows_and_cols=rows_and_cols,
             dd_pipe_like=False,
@@ -232,39 +235,39 @@ class FintabnetBuilder(DataFlowBaseBuilder):
         if build_mode == "table":
 
             @curry
-            def _crop_and_add_image(dp: Image, category_names: List[str]) -> Image:
+            def _crop_and_add_image(dp: Image, category_names: list[str]) -> Image:
                 return image_ann_to_image(dp, category_names=category_names)
 
             df = MapData(
                 df,
                 _crop_and_add_image(  # pylint: disable=E1120
                     category_names=[
-                        LayoutType.table,
-                        LayoutType.cell,
-                        CellType.header,
-                        CellType.body,
-                        TableType.item,
-                        LayoutType.row,
-                        LayoutType.column,
+                        LayoutType.TABLE,
+                        LayoutType.CELL,
+                        CellType.HEADER,
+                        CellType.BODY,
+                        TableType.ITEM,
+                        LayoutType.ROW,
+                        LayoutType.COLUMN,
                     ]
                 ),
             )
             df = MapData(
                 df,
                 maybe_ann_to_sub_image(  # pylint: disable=E1120  # 259
-                    category_names_sub_image=LayoutType.table,
+                    category_names_sub_image=LayoutType.TABLE,
                     category_names=[
-                        LayoutType.cell,
-                        CellType.header,
-                        CellType.body,
-                        TableType.item,
-                        LayoutType.row,
-                        LayoutType.column,
+                        LayoutType.CELL,
+                        CellType.HEADER,
+                        CellType.BODY,
+                        TableType.ITEM,
+                        LayoutType.ROW,
+                        LayoutType.COLUMN,
                     ],
                     add_summary=True,
                 ),
             )
-            df = MapData(df, lambda dp: [ann.image for ann in dp.get_annotation_iter(category_names=LayoutType.table)])
+            df = MapData(df, lambda dp: [ann.image for ann in dp.get_annotation(category_names=LayoutType.TABLE)])
             df = FlattenData(df)
             df = MapData(df, lambda dp: dp[0])
 
