@@ -37,30 +37,33 @@ Module for PubTables1M-Detection-PASCAL-VOC dataset. Install the dataset followi
     ├── PubTables-1M-Structure_Annotations_Test
     ├── PubTables-1M-Structure_Images_Test
 """
+from __future__ import annotations
 
 import os
 from typing import Mapping, Union
 
+from lazy_imports import try_import
+
 from ...dataflow import DataFlow, MapData, SerializerFiles
 from ...datasets.info import DatasetInfo
 from ...mapper.cats import filter_cat
 from ...mapper.maputils import curry
 from ...mapper.misc import xml_to_dict
 from ...mapper.pascalstruct import pascal_voc_dict_to_image
-from ...utils.detection_types import JsonDict
 from ...utils.file_utils import lxml_available
+from ...utils.fs import get_package_path
 from ...utils.settings import CellType, DatasetType, LayoutType
-from ...utils.systools import get_package_path
+from ...utils.types import JsonDict
 from ..base import _BuiltInDataset
 from ..dataflow_builder import DataFlowBaseBuilder
 from ..info import DatasetCategories
 from ..registry import dataset_registry
 
-if lxml_available():
+with try_import() as import_guard:
     from lxml import etree
 
 _NAME = "pubtables1m_det"
-
+_SHORT_DESCRIPTION = "PubTables1M is a dataset for table detection and structure recognition."
 _DESCRIPTION = (
     "[excerpt from Brandon Smock et. all. PubTables-1M: Towards Comprehensive Table Extraction From Unstructured \n"
     "Documents] '...we release PubTables1M, a dataset of nearly one million tables from PubMed Central Open Access \n"
@@ -77,14 +80,14 @@ _LICENSE = "Community Data License Agreement – Permissive, Version 1.0"
 _URL = "https://msropendata.com/datasets/505fcbe3-1383-42b1-913a-f651b8b712d3"
 
 _SPLITS: Mapping[str, str] = {"train": "train", "val": "val", "test": "test"}
-_TYPE = DatasetType.object_detection
+_TYPE = DatasetType.OBJECT_DETECTION
 _LOCATION = "PubTables1M"
 _ANNOTATION_FILES: Mapping[str, str] = {
     "train": "PubTables1M-Detection-PASCAL-VOC/train",
     "val": "PubTables1M-Detection-PASCAL-VOC/val",
     "test": "PubTables1M-Detection-PASCAL-VOC/test",
 }
-_INIT_CATEGORIES_DET = [LayoutType.table, LayoutType.table_rotated]
+_INIT_CATEGORIES_DET = [LayoutType.TABLE, LayoutType.TABLE_ROTATED]
 
 
 @dataset_registry.register("pubtables1m_det")
@@ -97,36 +100,47 @@ class Pubtables1MDet(_BuiltInDataset):
 
     @classmethod
     def _info(cls) -> DatasetInfo:
-        return DatasetInfo(name=_NAME, description=_DESCRIPTION, license=_LICENSE, url=_URL, splits=_SPLITS, type=_TYPE)
+        return DatasetInfo(
+            name=_NAME,
+            short_description=_SHORT_DESCRIPTION,
+            description=_DESCRIPTION,
+            license=_LICENSE,
+            url=_URL,
+            splits=_SPLITS,
+            type=_TYPE,
+        )
 
     def _categories(self) -> DatasetCategories:
         return DatasetCategories(init_categories=_INIT_CATEGORIES_DET)
 
-    def _builder(self) -> "Pubtables1MBuilder":
+    def _builder(self) -> Pubtables1MBuilder:
         return Pubtables1MBuilder(location=_LOCATION, annotation_files=_ANNOTATION_FILES)
 
 
 class Pubtables1MBuilder(DataFlowBaseBuilder):
     """
-    Pubtables1M dataflow builder
+    `Pubtables1M` dataflow builder
     """
 
     def build(self, **kwargs: Union[str, int]) -> DataFlow:
         """
-        Returns a dataflow from which you can stream datapoints of images. The following arguments affect the return
-        values of the dataflow:
+        Returns a dataflow from which you can stream datapoints of images. The following arguments affect the returns
+        of the dataflow:
 
-        `split:` Split of the dataset. Can be `train`, `val` or `test`. Default: `val`
+        Args:
+            kwargs:
+                (split) Split of the dataset. Can be `train`, `val` or `test`. Default: `val`
+                (max_datapoints) Will stop iterating after `max_datapoints`. Default: `None`
+                (load_image) Will load the image for each datapoint. Default: `False`
+                (fake_score) Will add a fake score so that annotations look like predictions. Default: `False`
 
-        `max_datapoints:` Will stop iterating after max_datapoints. Default: `None`
-
-        `load_image:` Will load the image for each datapoint.  Default: `False`
-
-        `fake_score:` Will add a fake score so that annotations look like predictions
-
-        :return: dataflow
+        Returns:
+            Dataflow
         """
 
+        if not lxml_available():
+            raise ModuleNotFoundError("Pubtables1MBuilder.build requires lxml but it is not installed.")
+
         split = str(kwargs.get("split", "val"))
         load_image = kwargs.get("load_image", False)
         max_datapoints = kwargs.get("max_datapoints")
@@ -174,7 +188,7 @@ class Pubtables1MBuilder(DataFlowBaseBuilder):
                 load_image,
                 filter_empty_image=True,
                 fake_score=fake_score,
-                category_name_mapping={"table": LayoutType.table, "table rotated": LayoutType.table_rotated},
+                category_name_mapping={"table": LayoutType.TABLE, "table rotated": LayoutType.TABLE_ROTATED},
             ),
         )
 
@@ -189,13 +203,13 @@ _ANNOTATION_FILES_STRUCT: Mapping[str, str] = {
 }
 
 _INIT_CATEGORIES_STRUCT = [
-    LayoutType.table,
-    LayoutType.row,
-    LayoutType.column,
-    CellType.spanning,
-    CellType.row_header,
-    CellType.column_header,
-    CellType.projected_row_header,
+    LayoutType.TABLE,
+    LayoutType.ROW,
+    LayoutType.COLUMN,
+    CellType.SPANNING,
+    CellType.ROW_HEADER,
+    CellType.COLUMN_HEADER,
+    CellType.PROJECTED_ROW_HEADER,
 ]
 
 _IMAGES: Mapping[str, str] = {
@@ -222,7 +236,7 @@ class Pubtables1MStruct(_BuiltInDataset):
     def _categories(self) -> DatasetCategories:
         return DatasetCategories(init_categories=_INIT_CATEGORIES_STRUCT)
 
-    def _builder(self) -> "Pubtables1MBuilderStruct":
+    def _builder(self) -> Pubtables1MBuilderStruct:
         return Pubtables1MBuilderStruct(location=_LOCATION, annotation_files=_ANNOTATION_FILES_STRUCT)
 
 
@@ -243,6 +257,8 @@ class Pubtables1MBuilderStruct(DataFlowBaseBuilder):
 
         :return: dataflow
         """
+        if not lxml_available():
+            raise ModuleNotFoundError("Pubtables1MBuilderStruct.build requires lxml but it is not installed.")
 
         split = str(kwargs.get("split", "val"))
         load_image = kwargs.get("load_image", False)
@@ -294,13 +310,13 @@ class Pubtables1MBuilderStruct(DataFlowBaseBuilder):
                 filter_empty_image=True,
                 fake_score=fake_score,
                 category_name_mapping={
-                    "table": LayoutType.table,
-                    "table spanning cell": CellType.spanning,
-                    "table row": LayoutType.row,
-                    "table row header": CellType.row_header,
-                    "table projected row header": CellType.projected_row_header,
-                    "table column": LayoutType.column,
-                    "table column header": CellType.column_header,
+                    "table": LayoutType.TABLE,
+                    "table spanning cell": CellType.SPANNING,
+                    "table row": LayoutType.ROW,
+                    "table row header": CellType.ROW_HEADER,
+                    "table projected row header": CellType.PROJECTED_ROW_HEADER,
+                    "table column": LayoutType.COLUMN,
+                    "table column header": CellType.COLUMN_HEADER,
                 },
             ),
         )
