@@ -29,24 +29,26 @@ Module for Publaynet dataset. Place the dataset as follows
     │ ├── train.txt
     │ ├── val.txt
 """
+from __future__ import annotations
 
 import os
 from typing import Mapping, Union
 
 from ...dataflow import DataFlow, MapData
 from ...dataflow.custom_serialize import SerializerTabsepFiles
-from ...datapoint.annotation import CategoryAnnotation, SummaryAnnotation
+from ...datapoint.annotation import CategoryAnnotation
 from ...datapoint.image import Image
 from ...mapper.cats import filter_summary
 from ...mapper.maputils import curry
 from ...utils.fs import load_image_from_file
-from ...utils.settings import DatasetType, DocumentType, PageType, TypeOrStr
+from ...utils.settings import DatasetType, DocumentType, PageType, SummaryType, TypeOrStr
 from ..base import _BuiltInDataset
 from ..dataflow_builder import DataFlowBaseBuilder
 from ..info import DatasetCategories, DatasetInfo
 from ..registry import dataset_registry
 
 _NAME = "rvl-cdip"
+_SHORT_DESCRIPTION = "RVL-CDIP is a dataset for document classification."
 _DESCRIPTION = (
     "The RVL-CDIP (Ryerson Vision Lab Complex Document Information Processing) dataset consists of 400, 000 gray- \n"
     "scale images in 16 classes, with 25, 000 images per class . There are 320, 000 training images, 40, 000  \n"
@@ -63,27 +65,27 @@ _LICENSE = (
 _URL = "https://www.cs.cmu.edu/~aharley/rvl-cdip/"
 
 _SPLITS: Mapping[str, str] = {"train": "train", "val": "val", "test": "test"}
-_TYPE = DatasetType.sequence_classification
+_TYPE = DatasetType.SEQUENCE_CLASSIFICATION
 _LOCATION = "rvl-cdip"
 
 _ANNOTATION_FILES: Mapping[str, str] = {"train": "labels/train.txt", "val": "labels/val.txt", "test": "labels/test.txt"}
 _INIT_CATEGORIES = [
-    DocumentType.letter,
-    DocumentType.form,
-    DocumentType.email,
-    DocumentType.handwritten,
-    DocumentType.advertisement,
-    DocumentType.scientific_report,
-    DocumentType.scientific_publication,
-    DocumentType.specification,
-    DocumentType.file_folder,
-    DocumentType.news_article,
-    DocumentType.budget,
-    DocumentType.invoice,
-    DocumentType.presentation,
-    DocumentType.questionnaire,
-    DocumentType.resume,
-    DocumentType.memo,
+    DocumentType.LETTER,
+    DocumentType.FORM,
+    DocumentType.EMAIL,
+    DocumentType.HANDWRITTEN,
+    DocumentType.ADVERTISEMENT,
+    DocumentType.SCIENTIFIC_REPORT,
+    DocumentType.SCIENTIFIC_PUBLICATION,
+    DocumentType.SPECIFICATION,
+    DocumentType.FILE_FOLDER,
+    DocumentType.NEWS_ARTICLE,
+    DocumentType.BUDGET,
+    DocumentType.INVOICE,
+    DocumentType.PRESENTATION,
+    DocumentType.QUESTIONNAIRE,
+    DocumentType.RESUME,
+    DocumentType.MEMO,
 ]
 
 
@@ -97,12 +99,20 @@ class Rvlcdip(_BuiltInDataset):
 
     @classmethod
     def _info(cls) -> DatasetInfo:
-        return DatasetInfo(name=_NAME, description=_DESCRIPTION, license=_LICENSE, url=_URL, splits=_SPLITS, type=_TYPE)
+        return DatasetInfo(
+            name=_NAME,
+            short_description=_SHORT_DESCRIPTION,
+            description=_DESCRIPTION,
+            license=_LICENSE,
+            url=_URL,
+            splits=_SPLITS,
+            type=_TYPE,
+        )
 
     def _categories(self) -> DatasetCategories:
         return DatasetCategories(init_categories=_INIT_CATEGORIES)
 
-    def _builder(self) -> "RvlcdipBuilder":
+    def _builder(self) -> RvlcdipBuilder:
         return RvlcdipBuilder(location=_LOCATION, annotation_files=_ANNOTATION_FILES)
 
 
@@ -113,16 +123,16 @@ class RvlcdipBuilder(DataFlowBaseBuilder):
 
     def build(self, **kwargs: Union[str, int]) -> DataFlow:
         """
-        Returns a dataflow from which you can stream datapoints of images. The following arguments affect the returns
-        of the dataflow:
+        Returns a dataflow from which you can stream datapoints of images.
 
-        `split:` Split of the dataset. Can be `train`, `val` or `test`. Default: `val`
+        Args:
+            kwargs:
+                split (str): Split of the dataset. Can be `train`, `val` or `test`. Default: `val`
+                max_datapoints (int): Will stop iterating after max_datapoints. Default: `None`
+                load_image (bool): Will load the image for each datapoint. Default: `False`
 
-        max_datapoints: Will stop iterating after max_datapoints. Default: `None`
-
-        load_image: Will load the image for each datapoint.  Default: `False`
-
-        :return: dataflow
+        Returns:
+            Dataflow
         """
 
         split = str(kwargs.get("split", "val"))
@@ -138,15 +148,15 @@ class RvlcdipBuilder(DataFlowBaseBuilder):
 
         @curry
         def _map_str_to_image(dp: str, load_img: bool) -> Image:
-            location, label = dp.split()[0], dp.split()[1]
-            label = str(int(label) + 1)
+            location, label_str = dp.split()[0], dp.split()[1]
+            label = int(label_str) + 1
             file_name = os.path.split(location)[1]
             image = Image(location=(self.get_workdir() / "images" / location).as_posix(), file_name=file_name)
             image.image = load_image_from_file(image.location)
-            summary = SummaryAnnotation()
+            summary = CategoryAnnotation(category_name=SummaryType.SUMMARY)
             categories_dict = self.categories.get_categories(init=True)
             summary.dump_sub_category(
-                PageType.document_type, CategoryAnnotation(category_name=categories_dict[label], category_id=str(label))
+                PageType.DOCUMENT_TYPE, CategoryAnnotation(category_name=categories_dict[label], category_id=label)
             )
             image.summary = summary
             if not load_img:
@@ -158,15 +168,14 @@ class RvlcdipBuilder(DataFlowBaseBuilder):
         if self.categories.is_filtered():
             df = MapData(
                 df,
-                filter_summary({PageType.document_type: self.categories.get_categories(as_dict=False, filtered=True)}),
+                filter_summary({PageType.DOCUMENT_TYPE: self.categories.get_categories(as_dict=False, filtered=True)}),
             )
 
             @curry
-            def _re_map_cat_ids(dp: Image, filtered_categories_name_as_key: Mapping[TypeOrStr, str]) -> Image:
-                if dp.summary:
-                    if PageType.document_type in dp.summary.sub_categories:
-                        summary_cat = dp.summary.get_sub_category(PageType.document_type)
-                        summary_cat.category_id = filtered_categories_name_as_key[summary_cat.category_name]
+            def _re_map_cat_ids(dp: Image, filtered_categories_name_as_key: Mapping[TypeOrStr, int]) -> Image:
+                if PageType.DOCUMENT_TYPE in dp.summary.sub_categories:
+                    summary_cat = dp.summary.get_sub_category(PageType.DOCUMENT_TYPE)
+                    summary_cat.category_id = filtered_categories_name_as_key[summary_cat.category_name]
                 return dp
 
             df = MapData(df, _re_map_cat_ids(self.categories.get_categories(filtered=True, name_as_key=True)))
