@@ -16,25 +16,25 @@
 # limitations under the License.
 
 """
-Module for saving
+Saving samples from a DataFlow
 """
 
 import json
+import os
 from pathlib import Path
 from typing import Optional
 
-from cv2 import imwrite
-
 from ..dataflow import DataFlow, MapData, SerializerJsonlines
 from ..datapoint.convert import convert_b64_to_np_array
 from ..datapoint.image import Image
-from ..utils.detection_types import JsonDict, Pathlike
 from ..utils.fs import mkdir_p
+from ..utils.types import ImageDict, PathLikeOrStr
+from ..utils.viz import viz_handler
 
 
 def dataflow_to_json(
     df: DataFlow,
-    path: Pathlike,
+    path: PathLikeOrStr,
     single_files: bool = False,
     file_name: Optional[str] = None,
     max_datapoints: Optional[int] = None,
@@ -42,20 +42,20 @@ def dataflow_to_json(
     highest_hierarchy_only: bool = False,
 ) -> None:
     """
-    Save a dataflow consisting of `datapoint.Image` to a jsonl file. Each image will be dumped into a separate
-    JSON object.
+    Save a dataflow consisting of `datapoint.Image` to a `jsonl` file. Each image will be dumped into a separate
+    `JSON` object.
 
-    :param df: Input dataflow
-    :param path: Path to save the file(s) to
-    :param single_files: will save image results to a single JSON file. If False all images of the dataflow will be d
-                         dumped into a single jsonl file.
-    :param file_name: file name, only needed for jsonl files
-    :param max_datapoints: Will stop saving after dumping max_datapoint images.
-    :param save_image_in_json: Will save the image to the JSON object
-    :param highest_hierarchy_only: If True it will remove all image attributes of ImageAnnotations
+    Args:
+        df: Input dataflow
+        path: Path to save the file(s) to
+        single_files: Will save image results to a single `JSON` file. If False all images of the dataflow will be
+                      dumped into a single `.jsonl` file.
+        file_name: file name, only needed for `jsonl` files
+        max_datapoints: Will stop saving after dumping max_datapoint images.
+        save_image_in_json: Will save the image to the `JSON` object
+        highest_hierarchy_only: If `True` it will remove all image attributes of `ImageAnnotation`s
     """
-    if isinstance(path, str):
-        path = Path(path)
+    path = Path(path)
     if single_files:
         mkdir_p(path)
     if not save_image_in_json:
@@ -63,14 +63,14 @@ def dataflow_to_json(
     if highest_hierarchy_only:
 
         def _remove_hh(dp: Image) -> Image:
-            dp.remove_image_from_lower_hierachy()
+            dp.remove_image_from_lower_hierarchy()
             return dp
 
         df = MapData(df, _remove_hh)
     df = MapData(df, lambda dp: dp.as_dict())
 
-    def _path_to_str(dp: JsonDict) -> JsonDict:
-        dp["location"] = str(dp["location"])
+    def _path_to_str(dp: ImageDict) -> ImageDict:
+        dp["location"] = os.fspath(dp["location"])
         return dp
 
     df = MapData(df, _path_to_str)
@@ -84,10 +84,11 @@ def dataflow_to_json(
                 target_file_png = path / "image" / (dp["file_name"].split(".")[0] + ".png")
                 image = dp.pop("_image")
                 image = convert_b64_to_np_array(image)
-                imwrite(str(target_file_png), image)
+
+                viz_handler.write_image(str(target_file_png), image)
 
             with open(target_file, "w", encoding="UTF-8") as file:
-                json.dump(dp, file)
+                json.dump(dp, file, indent=2)
 
     else:
         if not file_name:
