@@ -16,16 +16,14 @@
 # limitations under the License.
 
 """
-Init file for eval package. Contains metrics (customized for special tasks), evaluators and tensorpack related callbacks
-for training.
+## Evalulation metrics and Evaluators
+
+Contains metrics (customized for special tasks), evaluators and Tensorpack callback
 """
 
-from ..utils.file_utils import apted_available
 from .accmetric import *
 from .base import *
 from .cocometric import *
 from .eval import *
 from .registry import *
-
-if apted_available():
-    from .tedsmetric import *
+from .tedsmetric import *
