@@ -16,10 +16,9 @@
 # limitations under the License.
 
 """
-Wrappers for models of external libraries as well as implementation of the Cascade-RCNN model of Tensorpack.
+# Wrappers for models of external libraries
 """
 
-from ..utils.file_utils import tensorpack_available
 from .base import *
 from .d2detect import *
 from .deskew import *
@@ -31,6 +30,4 @@ from .model import *
 from .pdftext import *
 from .tessocr import *
 from .texocr import *  # type: ignore
-
-if tensorpack_available():
-    from .tpdetect import *
+from .tpdetect import *
