@@ -10,26 +10,29 @@ This file is modified from
 """
 
 
-import cv2
 import numpy as np
-from tensorpack.dataflow.imgaug import ImageAugmentor, ResizeTransform  # pylint: disable=E0401
+from lazy_imports import try_import
 
-from ....utils.file_utils import cocotools_available
+with try_import() as import_guard:
+    from tensorpack.dataflow.imgaug import ImageAugmentor, ResizeTransform  # pylint: disable=E0401
 
-if cocotools_available():
+with try_import() as cc_import_guard:
     import pycocotools.mask as coco_mask
 
+if not import_guard.is_successful():
+    from ....utils.mocks import ImageAugmentor
+
 
 class CustomResize(ImageAugmentor):
     """
     Try resizing the shortest edge to a certain number while avoiding the longest edge to exceed max_size.
     """
 
-    def __init__(self, short_edge_length, max_size, interp=cv2.INTER_LINEAR):
+    def __init__(self, short_edge_length, max_size, interp=1):
         """
         :param short_edge_length: a [min, max] interval from which to sample the shortest edge length.
         :param max_size: maximum allowed longest edge length.
-        :param interp: cv2 interpolation mode
+        :param interp: Interpolation mode. We use Tensorpack's internal `ResizeTransform`, that always requires OpenCV
         """
         super().__init__()
         if isinstance(short_edge_length, int):
