@@ -12,22 +12,30 @@ This file is modified from
 from contextlib import ExitStack, contextmanager
 
 import numpy as np
+from lazy_imports import try_import
 
 # pylint: disable=import-error
-import tensorflow as tf
-from tensorpack import tfv1
-from tensorpack.models import BatchNorm, Conv2D, MaxPooling, layer_register
-from tensorpack.tfutils import argscope
-from tensorpack.tfutils.varreplace import custom_getter_scope, freeze_variables
+
+with try_import() as import_guard:
+    import tensorflow as tf
+    from tensorpack import tfv1
+    from tensorpack.models import BatchNorm, Conv2D, MaxPooling, layer_register
+    from tensorpack.tfutils import argscope
+    from tensorpack.tfutils.varreplace import custom_getter_scope, freeze_variables
 
 # pylint: enable=import-error
 
+if not import_guard.is_successful():
+    from .....utils.mocks import layer_register
+
 
 @layer_register(log_shape=True)
-def GroupNorm(x, group=32, gamma_initializer=tf.constant_initializer(1.0)):
+def GroupNorm(x, group=32, gamma_initializer=None):
     """
     More code that reproduces the paper can be found at <https://github.com/ppwwyyxx/GroupNorm-reproduce/>.
     """
+    if gamma_initializer is None:
+        gamma_initializer = tf.constant_initializer(1.0)
     shape = x.get_shape().as_list()
     ndims = len(shape)
     assert ndims == 4, shape
@@ -63,7 +71,7 @@ def freeze_affine_getter(getter, *args, **kwargs):
     if name.endswith("/gamma") or name.endswith("/beta"):
         kwargs["trainable"] = False
         ret = getter(*args, **kwargs)
-        tf.add_to_collection(tf.GraphKeys.MODEL_VARIABLES, ret)
+        tf.add_to_collection(tf.GraphKeys.MODEL_VARIABLES, ret)  # pylint: disable=E1101
     else:
         ret = getter(*args, **kwargs)
     return ret
@@ -153,7 +161,7 @@ def get_norm(cfg, zero_init=False):
     return lambda x: norm(layer_name, x, gamma_initializer=tf.zeros_initializer() if zero_init else None)
 
 
-def resnet_shortcut(l, n_out, stride, activation=tf.identity):
+def resnet_shortcut(l, n_out, stride, activation=None):
     """
     Defining the skip connection in bottleneck
 
@@ -163,9 +171,11 @@ def resnet_shortcut(l, n_out, stride, activation=tf.identity):
     :param activation: An activation function
     :return: tf.Tensor
     """
+    if activation is None:
+        activation = tf.identity
     n_in = l.shape[1]
     if n_in != n_out:  # change dimension when channel is not the same
-        return Conv2D("convshortcut", l, n_out, 1, strides=stride, activation=activation)
+        return Conv2D("convshortcut", l, n_out, 1, strides=stride, activation=activation)  # pylint: disable=E1124
     return l
 
 
@@ -181,12 +191,12 @@ def resnet_bottleneck(l, ch_out, stride, cfg):
     """
     shortcut = l
 
-    l = Conv2D("conv1", l, ch_out, 1, strides=1)
+    l = Conv2D("conv1", l, ch_out, 1, strides=1)  # pylint: disable=E1124
     if stride == 2:
         l = tf.pad(l, [[0, 0], [0, 0], maybe_reverse_pad(cfg, 0, 1), maybe_reverse_pad(cfg, 0, 1)])
-        l = Conv2D("conv2", l, ch_out, 3, strides=2, padding="VALID")
+        l = Conv2D("conv2", l, ch_out, 3, strides=2, padding="VALID")  # pylint: disable=E1124
     else:
-        l = Conv2D("conv2", l, ch_out, 3, strides=stride)
+        l = Conv2D("conv2", l, ch_out, 3, strides=stride)  # pylint: disable=E1124
     if cfg.BACKBONE.NORM != "None":
         l = Conv2D("conv3", l, ch_out * 4, 1, activation=get_norm(cfg, zero_init=True))
     else:
@@ -263,9 +273,9 @@ def resnet_fpn_backbone(image, cfg):
             ),
         )
         l.set_shape([None, chan, None, None])
-        l = Conv2D("conv0", l, 64, 7, strides=2, padding="VALID")
+        l = Conv2D("conv0", l, 64, 7, strides=2, padding="VALID")  # pylint: disable=E1124
         l = tf.pad(l, [[0, 0], [0, 0], maybe_reverse_pad(cfg, 0, 1), maybe_reverse_pad(cfg, 0, 1)])
-        l = MaxPooling("pool0", l, 3, strides=2, padding="VALID")
+        l = MaxPooling("pool0", l, 3, strides=2, padding="VALID")  # pylint: disable=E1124
 
     bottleneck = resnet_bottleneck if cfg.BACKBONE.BOTTLENECK == "resnet" else resnext32x4d_bottleneck
     with backbone_scope(cfg=cfg, freeze=freeze_at > 1):
