@@ -8,21 +8,26 @@
 This file is modified from
 <https://github.com/tensorpack/tensorpack/blob/master/examples/FasterRCNN/predict.py>
 """
+from __future__ import annotations
 
 from typing import List
 
-import cv2
 import numpy as np
-from tensorpack.predict.base import OfflinePredictor  # pylint: disable=E0401
+from lazy_imports import try_import
 
-from ....utils.file_utils import scipy_available
 from ....utils.transform import InferenceResize
 from ...base import DetectionResult
 from .common import clip_boxes
 
-if scipy_available():
+with try_import() as import_guard:
+    from tensorpack.predict.base import OfflinePredictor  # pylint: disable=E0401
+
+with try_import() as sp_import_guard:
     from scipy import interpolate
 
+with try_import() as cv2_import_guard:
+    import cv2
+
 
 def _scale_box(box, scale):
     w_half = (box[2] - box[0]) * 0.5
@@ -79,6 +84,7 @@ def _paste_mask(box, mask, shape, mrcnn_accurate_paste):
 
     # rounding errors could happen here, because masks were not originally computed for this shape.
     # but it's hard to do better, because the network does not know the "original" scale
+
     mask = (cv2.resize(mask, (w, h)) > 0.5).astype("uint8")
     ret = np.zeros(shape, dtype="uint8")
     ret[y_0 : y_1 + 1, x_0 : x_1 + 1] = mask
