@@ -16,21 +16,21 @@
 # limitations under the License.
 
 """
-Contains pipeline components that can be plugged into each other and predictors that are invoked by their
- respective component.
+# Pipeline components and pipelines
 """
 
 from .anngen import *
 from .base import *
-from .cell import *
 from .common import *
 from .concurrency import *
 from .doctectionpipe import *
 from .language import *
 from .layout import *
 from .lm import *
+from .order import *
 from .refine import *
 from .registry import *
 from .segment import *
+from .sub_layout import *
 from .text import *
 from .transform import *
