@@ -16,53 +16,57 @@
 # limitations under the License.
 
 """
-Module for refining methods of table segmentation. The refining methods lead ultimately to a table structure which
-enables html table representations
+Refining methods for table segmentation. The refining methods lead ultimately to a table structure which enables
+HTML table representations.
 """
+from __future__ import annotations
+
 from collections import defaultdict
 from copy import copy
 from dataclasses import asdict
 from itertools import chain, product
-from typing import DefaultDict, List, Optional, Set, Tuple, Union
+from typing import DefaultDict, Optional, Sequence, Union
 
 import networkx as nx  # type: ignore
 
 from ..datapoint.annotation import ImageAnnotation
 from ..datapoint.box import merge_boxes
-from ..datapoint.image import Image
+from ..datapoint.image import Image, MetaAnnotation
 from ..extern.base import DetectionResult
 from ..mapper.maputils import MappingContextManager
-from ..utils.detection_types import JsonDict
-from ..utils.settings import CellType, LayoutType, Relationships, TableType, get_type
+from ..utils.error import ImageError
+from ..utils.settings import CellType, LayoutType, ObjectTypes, Relationships, TableType, get_type
 from .base import PipelineComponent
 from .registry import pipeline_component_registry
 
 __all__ = ["TableSegmentationRefinementService", "generate_html_string"]
 
 
-def tiles_to_cells(dp: Image, table: ImageAnnotation) -> List[Tuple[Tuple[int, int], str]]:
+def tiles_to_cells(dp: Image, table: ImageAnnotation) -> list[tuple[tuple[int, int], str]]:
     """
-    Creation of a table parquet: A table is divided into a tile parquet with the (number of rows) x
-    (the number of columns) tiles.
+    Creates a table parquet by dividing a table into a tile parquet with the number of rows x number of columns tiles.
     Each tile is assigned a list of cell ids that are occupied by the cell. No cells but one or more cells can be
     assigned per tile.
 
-    :param dp: Image
-    :param table: Table image annotation
-    :return: Image
+    Args:
+        dp: `Image`
+        table: `ImageAnnotation`
+
+    Returns:
+        A list of tuples with tile positions and cell annotation ids.
     """
 
-    cell_ann_ids = table.get_relationship(Relationships.child)
+    cell_ann_ids = table.get_relationship(Relationships.CHILD)
     cells = dp.get_annotation(
-        category_names=[LayoutType.cell, CellType.header, CellType.body], annotation_ids=cell_ann_ids
+        category_names=[LayoutType.CELL, CellType.HEADER, CellType.BODY], annotation_ids=cell_ann_ids
     )
     tile_to_cells = []
 
     for cell in cells:
-        row_number = int(cell.get_sub_category(CellType.row_number).category_id)
-        col_number = int(cell.get_sub_category(CellType.column_number).category_id)
-        rs = int(cell.get_sub_category(CellType.row_span).category_id)
-        cs = int(cell.get_sub_category(CellType.column_span).category_id)
+        row_number = cell.get_sub_category(CellType.ROW_NUMBER).category_id
+        col_number = cell.get_sub_category(CellType.COLUMN_NUMBER).category_id
+        rs = cell.get_sub_category(CellType.ROW_SPAN).category_id
+        cs = cell.get_sub_category(CellType.COLUMN_SPAN).category_id
         for k in range(rs):
             for l in range(cs):
                 assert cell.annotation_id is not None, cell.annotation_id
@@ -72,16 +76,19 @@ def tiles_to_cells(dp: Image, table: ImageAnnotation) -> List[Tuple[Tuple[int, i
 
 
 def connected_component_tiles(
-    tile_to_cell_list: List[Tuple[Tuple[int, int], str]]
-) -> Tuple[List[Set[Tuple[int, int]]], DefaultDict[Tuple[int, int], List[str]]]:
+    tile_to_cell_list: list[tuple[tuple[int, int], str]]
+) -> tuple[list[set[tuple[int, int]]], DefaultDict[tuple[int, int], list[str]]]:
     """
-    The assignment of bricks to their cell occupancy induces a graph, with bricks as corners and cell edges. Cells that
-    lie on top of several bricks connect the underlying bricks. The graph generated according to this procedure is
-    usually multiple connected. The related components and the tile/cell ids assignment are determined.
+    Assigns bricks to their cell occupancy, inducing a graph with bricks as nodes and cell edges. Cells that lie on
+    top of several bricks connect the underlying bricks. The graph generated is usually multiple connected. Determines
+    the related components and the tile/cell ids assignment.
 
-    :param tile_to_cell_list: List of tuples with tile position and cell ids
-    :return: List of set with tiles that belong to the same connected component and a dict with tiles as keys and
-             assigned list of cell ids as values.
+    Args:
+        tile_to_cell_list: List of tuples with tile position and cell ids.
+
+    Returns:
+        A tuple containing a list of sets with tiles that belong to the same connected component and a dict with tiles
+        as keys and assigned list of cell ids as values.
     """
     cell_to_tile_list = [(cell_position[1], cell_position[0]) for cell_position in tile_to_cell_list]
     cells = set(tup[0] for tup in cell_to_tile_list)
@@ -106,7 +113,7 @@ def connected_component_tiles(
     connected_components_tiles = []
 
     for component in connected_components_cell:
-        tiles: Set[Tuple[int, int]] = set()
+        tiles: set[tuple[int, int]] = set()
         for cell in component:
             tiles = tiles.union(set(cell_to_tile_dict[cell]))  # type: ignore
         connected_components_tiles.append(tiles)
@@ -114,7 +121,7 @@ def connected_component_tiles(
     return connected_components_tiles, tile_to_cell_dict
 
 
-def _missing_tile(inputs: Set[Tuple[int, int]]) -> Optional[Tuple[int, int]]:
+def _missing_tile(inputs: set[tuple[int, int]]) -> Optional[tuple[int, int]]:
     min_x, min_y, max_x, max_y = (
         min(a[0] for a in inputs),
         min(a[1] for a in inputs),
@@ -130,15 +137,15 @@ def _missing_tile(inputs: Set[Tuple[int, int]]) -> Optional[Tuple[int, int]]:
 
 
 def _find_component(
-    tile: Tuple[int, int], reduced_connected_tiles: List[Set[Tuple[int, int]]]
-) -> Optional[Set[Tuple[int, int]]]:
+    tile: tuple[int, int], reduced_connected_tiles: list[set[tuple[int, int]]]
+) -> Optional[set[tuple[int, int]]]:
     for comp in reduced_connected_tiles:
         if tile in comp:
             return comp
     return None
 
 
-def _merge_components(reduced_connected_tiles: List[Set[Tuple[int, int]]]) -> List[Set[Tuple[int, int]]]:
+def _merge_components(reduced_connected_tiles: list[set[tuple[int, int]]]) -> list[set[tuple[int, int]]]:
     new_reduced_connected_tiles = []
     for connected_tile in reduced_connected_tiles:
         out = _missing_tile(connected_tile)
@@ -160,17 +167,18 @@ def _merge_components(reduced_connected_tiles: List[Set[Tuple[int, int]]]) -> Li
     return new_reduced_connected_tiles
 
 
-def generate_rectangle_tiling(connected_components_tiles: List[Set[Tuple[int, int]]]) -> List[Set[Tuple[int, int]]]:
+def generate_rectangle_tiling(connected_components_tiles: list[set[tuple[int, int]]]) -> list[set[tuple[int, int]]]:
     """
-    The determined connected components imply that all cells have to be combined which are above a connected component.
-    In addition, however, it must also be taken into account that cells must be rectangular. This means that related
-    components have to be combined whose combined cells above do not create a rectangular tiling. All tiles are combined
-    in such a way that all cells above them combine to form a rectangular scheme.
+    Combines connected components so that all cells above them form a rectangular scheme. Ensures that all tiles are
+    combined in such a way that all cells above them combine to form a rectangular tiling.
 
-    :param connected_components_tiles: List of set with tiles that belong to the same connected component
-    :return: List of sets with tiles, the cells on top of which together form a rectangular scheme
+    Args:
+        connected_components_tiles: List of sets with tiles that belong to the same connected component.
+
+    Returns:
+        List of sets with tiles, the cells on top of which together form a rectangular scheme.
     """
-    rectangle_tiling: List[Set[Tuple[int, int]]] = []
+    rectangle_tiling: list[set[tuple[int, int]]] = []
     inputs = connected_components_tiles
 
     while rectangle_tiling != inputs:
@@ -182,25 +190,28 @@ def generate_rectangle_tiling(connected_components_tiles: List[Set[Tuple[int, in
 
 
 def rectangle_cells(
-    rectangle_tiling: List[Set[Tuple[int, int]]], tile_to_cell_dict: DefaultDict[Tuple[int, int], List[str]]
-) -> List[Set[str]]:
+    rectangle_tiling: list[set[tuple[int, int]]], tile_to_cell_dict: DefaultDict[tuple[int, int], list[str]]
+) -> list[set[str]]:
     """
-    All cells are determined that are located above combined connected components and form a rectangular scheme.
+    Determines all cells that are located above combined connected components and form a rectangular scheme.
 
-    :param rectangle_tiling: List of sets with tiles, the cells on top of which together form a rectangular scheme
-    :param tile_to_cell_dict: Dict with tiles as keys and assigned list of cell ids as values.
-    :return: List of set of cell ids that form a rectangular scheme
+    Args:
+        rectangle_tiling: List of sets with tiles, the cells on top of which together form a rectangular scheme.
+        tile_to_cell_dict: Dict with tiles as keys and assigned list of cell ids as values.
+
+    Returns:
+        List of sets of cell ids that form a rectangular scheme.
     """
-    rectangle_tiling_cells: List[Set[str]] = []
+    rectangle_tiling_cells: list[set[str]] = []
     for rect_tiling_component in rectangle_tiling:
-        rect_cell_component: Set[str] = set()
+        rect_cell_component: set[str] = set()
         for el in rect_tiling_component:
             rect_cell_component = rect_cell_component.union(set(tile_to_cell_dict[el]))
         rectangle_tiling_cells.append(rect_cell_component)
     return rectangle_tiling_cells
 
 
-def _tiling_to_cell_position(inputs: Set[Tuple[int, int]]) -> Tuple[int, int, int, int]:
+def _tiling_to_cell_position(inputs: set[tuple[int, int]]) -> tuple[int, int, int, int]:
     row_number = min(a[0] for a in inputs)
     col_number = min(a[1] for a in inputs)
     row_span = max(abs(a[0] - b[0]) + 1 for a in inputs for b in inputs)
@@ -209,10 +220,17 @@ def _tiling_to_cell_position(inputs: Set[Tuple[int, int]]) -> Tuple[int, int, in
 
 
 def _html_cell(
-    cell_position: Union[Tuple[int, int, int, int], Tuple[()]], position_filled_list: List[Tuple[int, int]]
-) -> List[str]:
+    cell_position: Union[tuple[int, int, int, int], tuple[()]], position_filled_list: list[tuple[int, int]]
+) -> list[str]:
     """
-    Html table cell string generation
+    Generates an HTML table cell string.
+
+    Args:
+        cell_position: Cell position tuple or empty tuple.
+        position_filled_list: List of filled positions.
+
+    Returns:
+        List of HTML strings representing the cell.
     """
     html = ["<td"]
     if not cell_position:
@@ -237,14 +255,24 @@ def _html_cell(
 
 
 def _html_row(
-    row_list: List[Tuple[int, int, int, int]],
-    position_filled_list: List[Tuple[int, int]],
+    row_list: list[tuple[int, int, int, int]],
+    position_filled_list: list[tuple[int, int]],
     this_row: int,
     number_of_cols: int,
-    row_ann_id_list: List[str],
-) -> List[str]:
+    row_ann_id_list: list[str],
+) -> list[str]:
     """
-    Html table row string generation
+    Generates an HTML table row string.
+
+    Args:
+        row_list: List of cell position tuples for the row.
+        position_filled_list: List of filled positions.
+        this_row: The current row number.
+        number_of_cols: The total number of columns.
+        row_ann_id_list: List of annotation ids for the row.
+
+    Returns:
+        List of HTML strings representing the row.
     """
     html = ["<tr>"]
     for idx in range(1, number_of_cols + 1):
@@ -274,16 +302,25 @@ def _html_row(
 
 
 def _html_table(
-    table_list: List[Tuple[int, List[Tuple[int, int, int, int]]]],
-    cells_ann_list: List[Tuple[int, List[str]]],
+    table_list: list[tuple[int, list[tuple[int, int, int, int]]]],
+    cells_ann_list: list[tuple[int, list[str]]],
     number_of_rows: int,
     number_of_cols: int,
-) -> List[str]:
+) -> list[str]:
     """
-    Html table string generation
+    Generates an HTML table string.
+
+    Args:
+        table_list: List of tuples with row number and list of cell position tuples.
+        cells_ann_list: List of tuples with row number and list of annotation ids.
+        number_of_rows: The total number of rows.
+        number_of_cols: The total number of columns.
+
+    Returns:
+        List of HTML strings representing the table.
     """
     html = ["<table>"]
-    position_filled: List[Tuple[int, int]] = []
+    position_filled: list[tuple[int, int]] = []
     for idx in range(1, number_of_rows + 1):
         row_idx = list(filter(lambda x: x[0] == idx, table_list))[0][1]  # pylint:disable=W0640
         row_ann_ids = list(filter(lambda x: x[0] == idx, cells_ann_list))[0][1]  # pylint:disable=W0640
@@ -293,49 +330,49 @@ def _html_table(
     return html
 
 
-def generate_html_string(table: ImageAnnotation) -> List[str]:
+def generate_html_string(table: ImageAnnotation, cell_names: Sequence[ObjectTypes]) -> list[str]:
     """
-    Takes the table segmentation by using table cells row number, column numbers etc. and generates a html
-    representation.
+    Generates an HTML representation of a table using table segmentation by row number, column number, etc.
 
-    :param table: An annotation that has a not None image and fully segmented cell annotation.
-    :return: HTML representation of the table
+    Note:
+        It must be ensured that all cells have a row number, column number, row span, and column span, and that the
+        dissection by rows and columns is completely covered by cells.
+
+    Args:
+        table: An annotation that has a not None image and fully segmented cell annotation.
+        cell_names: List of cell names that are used for the table segmentation.
+
+    Returns:
+        HTML representation of the table.
+
+    Raises:
+        `ImageError`: If `table.image` is None.
     """
     if table.image is None:
-        raise ValueError("table.image cannot be None")
+        raise ImageError("table.image cannot be None")
     table_image = table.image
-    cells = table_image.get_annotation(
-        category_names=[
-            LayoutType.cell,
-            CellType.header,
-            CellType.body,
-            CellType.spanning,
-            CellType.row_header,
-            CellType.column_header,
-            CellType.projected_row_header,
-        ]
-    )
-    number_of_rows = int(table_image.summary.get_sub_category(TableType.number_of_rows).category_id)
-    number_of_cols = int(table_image.summary.get_sub_category(TableType.number_of_columns).category_id)
+    cells = table_image.get_annotation(category_names=cell_names)
+    number_of_rows = table_image.summary.get_sub_category(TableType.NUMBER_OF_ROWS).category_id
+    number_of_cols = table_image.summary.get_sub_category(TableType.NUMBER_OF_COLUMNS).category_id
     table_list = []
     cells_ann_list = []
     for row_number in range(1, number_of_rows + 1):
         cells_of_row = list(
             sorted(
                 filter(
-                    lambda cell: cell.get_sub_category(CellType.row_number).category_id
-                    == str(row_number),  # pylint: disable=W0640
+                    lambda cell: cell.get_sub_category(CellType.ROW_NUMBER).category_id
+                    == row_number,  # pylint: disable=W0640
                     cells,
                 ),
-                key=lambda cell: cell.get_sub_category(CellType.column_number).category_id,
+                key=lambda cell: cell.get_sub_category(CellType.COLUMN_NUMBER).category_id,
             )
         )
         row_list = [
             (
-                int(cell.get_sub_category(CellType.row_number).category_id),
-                int(cell.get_sub_category(CellType.column_number).category_id),
-                int(cell.get_sub_category(CellType.row_span).category_id),
-                int(cell.get_sub_category(CellType.column_span).category_id),
+                cell.get_sub_category(CellType.ROW_NUMBER).category_id,
+                cell.get_sub_category(CellType.COLUMN_NUMBER).category_id,
+                cell.get_sub_category(CellType.ROW_SPAN).category_id,
+                cell.get_sub_category(CellType.COLUMN_SPAN).category_id,
             )
             for cell in cells_of_row
         ]
@@ -360,12 +397,11 @@ class TableSegmentationRefinementService(PipelineComponent):
     | C3   C3  |
     +----------+
 
-    The first two cells have the same column assignment via the segmentation and must therefore be merged.
-    Note that the number of rows and columns does not change in the refinement process. What changes is just the number
-    of cells.
+    The first two cells have the same column assignment via the segmentation and must therefore be merged. Note that
+    the number of rows and columns does not change in the refinement process. What changes is just the number of cells.
 
-    Furthermore, when merging, it must be ensured that the combined cells still have a rectangular shape.
-    This is also guaranteed in the refining process.
+    Furthermore, when merging, it must be ensured that the combined cells still have a rectangular shape. This is also
+    guaranteed in the refining process.
 
     +----------+
     | C1 |  C2 |
@@ -373,40 +409,47 @@ class TableSegmentationRefinementService(PipelineComponent):
     | C3 |  C3 |
     +----------+
 
-    The table consists of one row and two columns. The upper cells belong together with the lower cell.
-    However, this means that all cells must be merged with one another so that the table only consists of one cell
-    after the refinement process.
+    The table consists of one row and two columns. The upper cells belong together with the lower cell. However, this
+    means that all cells must be merged with one another so that the table only consists of one cell after the
+    refinement process.
 
-    **Example**
+    Example:
+        ```python
+        layout = ImageLayoutService(layout_detector, to_image=True, crop_image=True)
+        cell = SubImageLayoutService(cell_detector, "TABLE")
+        row_col = SubImageLayoutService(row_col_detector, "TABLE")
+        table_segmentation = TableSegmentationService("ioa",0.9,0.8,True,0.0001,0.0001)
+        table_segmentation_refinement = TableSegmentationRefinementService()
 
-            layout = ImageLayoutService(layout_detector, to_image=True, crop_image=True)
-            cell = SubImageLayoutService(cell_detector, "TABLE")
-            row_col = SubImageLayoutService(row_col_detector, "TABLE")
-            table_segmentation = TableSegmentationService("ioa",0.9,0.8,True,0.0001,0.0001)
-            table_segmentation_refinement = TableSegmentationRefinementService()
-
-            table_recognition_pipe = DoctectionPipe([layout,
-                                                     cell,
-                                                     row_col,
-                                                     table_segmentation,
-                                                     table_segmentation_refinement])
-            df = pipe.analyze(path="path/to/document.pdf")
-
-            for dp in df:
-                ...
+        table_recognition_pipe = DoctectionPipe([layout,
+                                                 cell,
+                                                 row_col,
+                                                 table_segmentation,
+                                                 table_segmentation_refinement])
+        df = pipe.analyze(path="path/to/document.pdf")
 
+        for dp in df:
+            ...
+        ```
     """
 
-    def __init__(self) -> None:
-        self._table_name = LayoutType.table
-        self._cell_names = [CellType.header, CellType.body, LayoutType.cell]
+    def __init__(self, table_names: Sequence[ObjectTypes], cell_names: Sequence[ObjectTypes]) -> None:
+        """
+        Initializes the `TableSegmentationRefinementService`.
+
+        Args:
+            table_names: Sequence of table object types.
+            cell_names: Sequence of cell object types.
+        """
+        self.table_name = table_names
+        self.cell_names = cell_names
         super().__init__("table_segment_refine")
 
     def serve(self, dp: Image) -> None:
-        tables = dp.get_annotation(category_names=self._table_name)
+        tables = dp.get_annotation(category_names=self.table_name)
         for table in tables:
             if table.image is None:
-                raise ValueError("table.image cannot be None")
+                raise ImageError("table.image cannot be None")
             tiles_to_cells_list = tiles_to_cells(dp, table)
             connected_components, tile_to_cell_dict = connected_component_tiles(tiles_to_cells_list)
             rectangle_tiling = generate_rectangle_tiling(connected_components)
@@ -415,30 +458,28 @@ class TableSegmentationRefinementService(PipelineComponent):
                 no_context_error = True
                 if len(cells_to_merge) != 1:
                     cells = dp.get_annotation(annotation_ids=list(cells_to_merge))
-                    cell_boxes = [
-                        cell.image.get_embedding(table.image.image_id) for cell in cells if cell.image is not None
-                    ]
+                    cell_boxes = [cell.get_bounding_box(table.image.image_id) for cell in cells]
                     merged_box = merge_boxes(*cell_boxes)
                     det_result = DetectionResult(
                         box=merged_box.to_list(mode="xyxy"),
                         score=-1.0,
-                        class_id=int(cells[0].category_id),
+                        class_id=cells[0].category_id,
                         class_name=get_type(cells[0].category_name),
                     )
                     new_cell_ann_id = self.dp_manager.set_image_annotation(det_result, table.annotation_id)
                     if new_cell_ann_id is not None:
                         row_number, col_number, row_span, col_span = _tiling_to_cell_position(tiling)
                         self.dp_manager.set_category_annotation(
-                            CellType.row_number, row_number, CellType.row_number, new_cell_ann_id
+                            CellType.ROW_NUMBER, row_number, CellType.ROW_NUMBER, new_cell_ann_id
                         )
                         self.dp_manager.set_category_annotation(
-                            CellType.column_number, col_number, CellType.column_number, new_cell_ann_id
+                            CellType.COLUMN_NUMBER, col_number, CellType.COLUMN_NUMBER, new_cell_ann_id
                         )
                         self.dp_manager.set_category_annotation(
-                            CellType.row_span, row_span, CellType.row_span, new_cell_ann_id
+                            CellType.ROW_SPAN, row_span, CellType.ROW_SPAN, new_cell_ann_id
                         )
                         self.dp_manager.set_category_annotation(
-                            CellType.column_span, col_span, CellType.column_span, new_cell_ann_id
+                            CellType.COLUMN_SPAN, col_span, CellType.COLUMN_SPAN, new_cell_ann_id
                         )
                     else:
                         # DetectionResult cannot be dumped, hence merged_box must already exist. Hence, it must
@@ -453,50 +494,59 @@ class TableSegmentationRefinementService(PipelineComponent):
                         for cell in cells:
                             cell.deactivate()
 
-            cells = table.image.get_annotation(category_names=self._cell_names)
-            number_of_rows = max([int(cell.get_sub_category(CellType.row_number).category_id) for cell in cells])
-            number_of_cols = max([int(cell.get_sub_category(CellType.column_number).category_id) for cell in cells])
-            max_row_span = max([int(cell.get_sub_category(CellType.row_span).category_id) for cell in cells])
-            max_col_span = max([int(cell.get_sub_category(CellType.column_span).category_id) for cell in cells])
+            cells = table.image.get_annotation(category_names=self.cell_names)
+            number_of_rows = max(cell.get_sub_category(CellType.ROW_NUMBER).category_id for cell in cells)
+            number_of_cols = max(cell.get_sub_category(CellType.COLUMN_NUMBER).category_id for cell in cells)
+            max_row_span = max(cell.get_sub_category(CellType.ROW_SPAN).category_id for cell in cells)
+            max_col_span = max(cell.get_sub_category(CellType.COLUMN_SPAN).category_id for cell in cells)
             # TODO: the summaries should be sub categories of the underlying ann
+            if (
+                TableType.NUMBER_OF_ROWS in table.image.summary.sub_categories
+                and TableType.NUMBER_OF_COLUMNS in table.image.summary.sub_categories
+                and TableType.MAX_ROW_SPAN in table.image.summary.sub_categories
+                and TableType.MAX_COL_SPAN in table.image.summary.sub_categories
+            ):
+                table.image.summary.remove_sub_category(TableType.NUMBER_OF_ROWS)
+                table.image.summary.remove_sub_category(TableType.NUMBER_OF_COLUMNS)
+                table.image.summary.remove_sub_category(TableType.MAX_ROW_SPAN)
+                table.image.summary.remove_sub_category(TableType.MAX_COL_SPAN)
+
             self.dp_manager.set_summary_annotation(
-                TableType.number_of_rows, TableType.number_of_rows, number_of_rows, annotation_id=table.annotation_id
+                TableType.NUMBER_OF_ROWS, TableType.NUMBER_OF_ROWS, number_of_rows, annotation_id=table.annotation_id
             )
             self.dp_manager.set_summary_annotation(
-                TableType.number_of_columns,
-                TableType.number_of_columns,
+                TableType.NUMBER_OF_COLUMNS,
+                TableType.NUMBER_OF_COLUMNS,
                 number_of_cols,
                 annotation_id=table.annotation_id,
             )
             self.dp_manager.set_summary_annotation(
-                TableType.max_row_span, TableType.max_row_span, max_row_span, annotation_id=table.annotation_id
+                TableType.MAX_ROW_SPAN, TableType.MAX_ROW_SPAN, max_row_span, annotation_id=table.annotation_id
             )
             self.dp_manager.set_summary_annotation(
-                TableType.max_col_span, TableType.max_col_span, max_col_span, annotation_id=table.annotation_id
+                TableType.MAX_COL_SPAN, TableType.MAX_COL_SPAN, max_col_span, annotation_id=table.annotation_id
             )
-            html = generate_html_string(table)
-            self.dp_manager.set_container_annotation(TableType.html, -1, TableType.html, table.annotation_id, html)
+            html = generate_html_string(table, self.cell_names)
+            self.dp_manager.set_container_annotation(TableType.HTML, -1, TableType.HTML, table.annotation_id, html)
 
-    def clone(self) -> PipelineComponent:
-        return self.__class__()
+    def clone(self) -> TableSegmentationRefinementService:
+        return self.__class__(self.table_name, self.cell_names)
 
-    def get_meta_annotation(self) -> JsonDict:
-        return dict(
-            [
-                ("image_annotations", []),
-                (
-                    "sub_categories",
-                    {
-                        LayoutType.cell: {
-                            CellType.row_number,
-                            CellType.column_number,
-                            CellType.row_span,
-                            CellType.column_span,
-                        },
-                        LayoutType.table: {TableType.html},
-                    },
-                ),
-                ("relationships", {}),
-                ("summaries", []),
-            ]
+    def get_meta_annotation(self) -> MetaAnnotation:
+        return MetaAnnotation(
+            image_annotations=(),
+            sub_categories={
+                LayoutType.CELL: {
+                    CellType.ROW_NUMBER: {CellType.ROW_NUMBER},
+                    CellType.COLUMN_NUMBER: {CellType.COLUMN_NUMBER},
+                    CellType.ROW_SPAN: {CellType.ROW_SPAN},
+                    CellType.COLUMN_SPAN: {CellType.COLUMN_SPAN},
+                },
+                LayoutType.TABLE: {TableType.HTML: {TableType.HTML}},
+            },
+            relationships={},
+            summaries=(),
         )
+
+    def clear_predictor(self) -> None:
+        pass
