@@ -16,31 +16,13 @@
 # limitations under the License.
 
 """
-Module for training Tensorpack `GeneralizedRCNN`
+Training Tensorpack's `GeneralizedRCNN`
 """
 
 import os
-from typing import Dict, List, Optional, Sequence, Type, Union
+from typing import Optional, Sequence, Type, Union
 
-# pylint: disable=import-error
-from tensorpack.callbacks import (
-    EstimatedTimeLeft,
-    GPUMemoryTracker,
-    GPUUtilizationTracker,
-    HostMemoryTracker,
-    ModelSaver,
-    PeriodicCallback,
-    ScheduledHyperParamSetter,
-    SessionRunTimeout,
-    ThroughputTracker,
-)
-
-# todo: check how dataflow import is directly possible without having AssertionError
-from tensorpack.dataflow import ProxyDataFlow, imgaug
-from tensorpack.input_source import QueueInput
-from tensorpack.tfutils import SmartInit
-from tensorpack.train import SyncMultiGPUTrainerReplicated, TrainConfig, launch_train_with_config
-from tensorpack.utils import logger
+from lazy_imports import try_import
 
 from ..dataflow.base import DataFlow
 from ..dataflow.common import MapData
@@ -58,22 +40,44 @@ from ..extern.tp.tpfrcnn.preproc import anchors_and_labels, augment
 from ..extern.tpdetect import TPFrcnnDetector
 from ..mapper.maputils import LabelSummarizer
 from ..mapper.tpstruct import image_to_tp_frcnn_training
-from ..pipe.base import PredictorPipelineComponent
 from ..pipe.registry import pipeline_component_registry
-from ..utils.detection_types import JsonDict
 from ..utils.file_utils import set_mp_spawn
 from ..utils.fs import get_load_image_func
 from ..utils.logger import log_once
 from ..utils.metacfg import AttrDict, set_config_by_yaml
 from ..utils.tqdm import get_tqdm
+from ..utils.types import JsonDict, PathLikeOrStr
 from ..utils.utils import string_to_dict
 
+with try_import() as tp_import_guard:
+    # todo: check how dataflow import is directly possible without having an AssertionError
+    # pylint: disable=import-error
+    from tensorpack.callbacks import (
+        EstimatedTimeLeft,
+        GPUMemoryTracker,
+        GPUUtilizationTracker,
+        HostMemoryTracker,
+        ModelSaver,
+        PeriodicCallback,
+        ScheduledHyperParamSetter,
+        SessionRunTimeout,
+        ThroughputTracker,
+    )
+    from tensorpack.dataflow import ProxyDataFlow, imgaug
+    from tensorpack.input_source import QueueInput
+    from tensorpack.tfutils import SmartInit
+    from tensorpack.train import SyncMultiGPUTrainerReplicated, TrainConfig, launch_train_with_config
+    from tensorpack.utils import logger
+
 __all__ = ["train_faster_rcnn"]
 
 
 class LoadAugmentAddAnchors:
     """
     A helper class for default mapping `load_augment_add_anchors`.
+
+    Args:
+        config: An `AttrDict` configuration for TP FRCNN.
     """
 
     def __init__(self, config: AttrDict) -> None:
@@ -88,9 +92,15 @@ def load_augment_add_anchors(dp: JsonDict, config: AttrDict) -> Optional[JsonDic
     Transforming an image before entering the graph. This function bundles all the necessary steps to feed
     the network for training.
 
-    :param dp: A dict with 'file_name', 'gt_boxes', 'gt_labels' and optional 'image'
-    :param config: An `AttrDict` with a TP frcnn config
-    :return: An dict with all necessary keys for feeding the graph
+    Args:
+        dp: A dict with `file_name`, `gt_boxes`, `gt_labels` and optional `image`.
+        config: An `AttrDict` with a TP frcnn config.
+
+    Returns:
+        A dict with all necessary keys for feeding the graph.
+
+    Note:
+        If `image` is not in `dp`, it will be loaded from `file_name`.
     """
     cfg = config
     if "image" not in dp:
@@ -123,14 +133,20 @@ def get_train_dataflow(
     dataset: DatasetBase, config: AttrDict, use_multi_proc_for_train: bool, **build_train_kwargs: str
 ) -> DataFlow:
     """
-    Return a dataflow for training TP Frcnn. The returned dataflow depends on the dataset and the configuration of
+    Return a dataflow for training TP FRCNN. The returned dataflow depends on the dataset and the configuration of
     the model, as the augmentation is part of the data preparation.
 
-    :param dataset: A dataset for object detection
-    :param config: An `AttrDict` with a TP Frcnn config
-    :param use_multi_proc_for_train: If set to `True` will use multi processes for augmenting
-    :param build_train_kwargs: build configuration of the dataflow.
-    :return: A dataflow
+    Args:
+        dataset: A dataset for object detection.
+        config: An `AttrDict` with a TP FRCNN config.
+        use_multi_proc_for_train: If set to `True` will use multi processes for augmenting.
+        build_train_kwargs: Build configuration of the dataflow.
+
+    Returns:
+        A dataflow.
+
+    Note:
+        If `use_multi_proc_for_train` is `True`, multi-processing will be used for augmentation.
     """
 
     set_mp_spawn()
@@ -159,7 +175,7 @@ def get_train_dataflow(
                 )
             summarizer.dump(dp["gt_labels"])
             datapoints.append(dp)
-            status_bar.update()  # type: ignore
+            status_bar.update()
     summarizer.print_summary_histogram(dd_logic=False)
     num_datapoints = len(datapoints)
     logger.info("Total #images for training: %i", num_datapoints)
@@ -183,11 +199,11 @@ def get_train_dataflow(
 
 
 def train_faster_rcnn(
-    path_config_yaml: str,
+    path_config_yaml: PathLikeOrStr,
     dataset_train: DatasetBase,
-    path_weights: str = "",
-    config_overwrite: Optional[List[str]] = None,
-    log_dir: str = "train_log/frcnn",
+    path_weights: PathLikeOrStr,
+    config_overwrite: Optional[list[str]] = None,
+    log_dir: PathLikeOrStr = "train_log/frcnn",
     build_train_config: Optional[Sequence[str]] = None,
     dataset_val: Optional[DatasetBase] = None,
     build_val_config: Optional[Sequence[str]] = None,
@@ -201,34 +217,46 @@ def train_faster_rcnn(
     Train Faster-RCNN from Scratch or fine-tune a model using Tensorpack's training API. Observe the training with
     Tensorpack callbacks and evaluate the training progress with a validation data set after certain training intervals.
 
-    Tensorpack provides a training API under TF1. Training runs under a TF2 installation if TF2 behavior is deactivated.
+    Info:
+        Tensorpack provides a training API under TF1. Training runs under a TF2 installation if TF2 behavior is
+        deactivated.
 
-    :param path_config_yaml: path to TP config file. Check the
-                             [deepdoctection.extern.tp.tpfrcnn.config.config][] for various settings.
-    :param dataset_train: the dataset to use for training.
-    :param path_weights: path to a checkpoint, if you want to continue training or fine-tune. Will train from scratch if
-                         nothing is passed.
-    :param config_overwrite: Pass a list of arguments if some configs from the .yaml file should be replaced. Use the
-                             list convention, e.g. ['TRAIN.STEPS_PER_EPOCH=500', 'OUTPUT.RESULT_SCORE_THRESH=0.4']
-    :param log_dir: Path to log dir. Will default to TRAIN.LOG_DIR
-    :param build_train_config: dataflow build setting. Again, use list convention setting, e.g. ['max_datapoints=1000']
-    :param dataset_val: the dataset to use for validation.
-    :param build_val_config: same as 'build_train_config' but for validation
-    :param metric_name: A metric name to choose for validation. Will use the default setting. If you want a custom
-                        metric setting pass a metric explicitly.
-    :param metric: A metric to choose for validation.
-    :param pipeline_component_name: A pipeline component to use for validation.
+    Args:
+        path_config_yaml: Path to TP config file. Check the `deepdoctection.extern.tp.tpfrcnn.config.config` for various
+                          settings.
+        dataset_train: The dataset to use for training.
+        path_weights: Path to a checkpoint, if you want to continue training or fine-tune. Will train from scratch if
+                      nothing is passed.
+        config_overwrite: Pass a list of arguments if some configs from the .yaml file should be replaced. Use the list
+                          convention, e.g. `[`TRAIN.STEPS_PER_EPOCH=500`, `OUTPUT.RESULT_SCORE_THRESH=0.4`]`.
+        log_dir: Path to log dir. Will default to `TRAIN.LOG_DIR`.
+        build_train_config: Dataflow build setting. Use list convention setting, e.g. `[`max_datapoints=1000`]`.
+        dataset_val: The dataset to use for validation.
+        build_val_config: Same as `build_train_config` but for validation.
+        metric_name: A metric name to choose for validation. Will use the default setting. If you want a custom metric
+                     setting pass a metric explicitly.
+        metric: A metric to choose for validation.
+        pipeline_component_name: A pipeline component to use for validation.
+
+    Example:
+        ```python
+        train_faster_rcnn(
+            path_config_yaml="config.yaml",
+            dataset_train=my_train_dataset,
+            path_weights="weights.ckpt"
+        )
+        ```
     """
 
     assert disable_tfv2()  # TP works only in Graph mode
 
-    build_train_dict: Dict[str, str] = {}
+    build_train_dict: dict[str, str] = {}
     if build_train_config is not None:
         build_train_dict = string_to_dict(",".join(build_train_config))
     if "split" not in build_train_dict:
         build_train_dict["split"] = "train"
 
-    build_val_dict: Dict[str, str] = {}
+    build_val_dict: dict[str, str] = {}
     if build_val_config is not None:
         build_val_dict = string_to_dict(",".join(build_val_config))
     if "split" not in build_val_dict:
@@ -236,13 +264,14 @@ def train_faster_rcnn(
 
     config_overwrite = [] if config_overwrite is None else config_overwrite
 
-    log_dir = "TRAIN.LOG_DIR=" + log_dir
+    log_dir = "TRAIN.LOG_DIR=" + os.fspath(log_dir)
     config_overwrite.append(log_dir)
 
     config = set_config_by_yaml(path_config_yaml)
-
+    config.freeze(False)
     if config_overwrite:
         config.update_args(config_overwrite)
+    config.freeze(True)
 
     categories = dataset_train.dataflow.categories.get_categories(filtered=True)
     model_frcnn_config(config, categories, False)
@@ -297,7 +326,6 @@ def train_faster_rcnn(
         )  # only a wrapper for the predictor itself. Will be replaced in Callback
         pipeline_component_cls = pipeline_component_registry.get(pipeline_component_name)
         pipeline_component = pipeline_component_cls(detector)
-        assert isinstance(pipeline_component, PredictorPipelineComponent)
         category_names = list(categories.values())
         callbacks.extend(
             [
@@ -308,6 +336,7 @@ def train_faster_rcnn(
                     metric,  # type: ignore
                     pipeline_component,
                     *model.get_inference_tensor_names(),  # type: ignore
+                    cfg=detector.model.cfg,
                     **build_val_dict
                 )
             ]
