@@ -3,23 +3,26 @@
 
 # Copyright (c) Tensorpack Contributors
 # Licensed under the Apache License, Version 2.0 (the "License")
+
 """
 This file is modified from
-https://github.com/tensorpack/tensorpack/blob/master/tensorpack/utils/logger.py
+<https://github.com/tensorpack/tensorpack/blob/master/tensorpack/utils/logger.py>
 
 The logger module itself has the common logging functions of Python's
 `logging.Logger`.
 
-**Example:**
+Example:
+    ```python
+    from deepdoctection.utils.logger import logger
 
-    ..code-block::python
+    logger.set_logger_dir("path/to/dir")
+    logger.info("Something has happened")
+    logger.warning("Attention!")
+    logger.error("Error happened!")
+    ```
 
-        from deepdoctection.utils.logger import logger
-
-        logger.set_logger_dir("path/to/dir")
-        logger.info("Something has happened")
-        logger.warning("Attention!")
-        logger.error("Error happened!")
+Log levels can be set via the environment variable `LOG_LEVEL` (default: INFO).
+`STD_OUT_VERBOSE` will print a verbose message to the terminal (default: False).
 """
 
 import errno
@@ -30,44 +33,75 @@ import logging.config
 import os
 import shutil
 import sys
+from dataclasses import dataclass, field
 from datetime import datetime
 from pathlib import Path
-from typing import Any, Dict, Optional, no_type_check
+from typing import Any, Optional, Union, no_type_check
 
 from termcolor import colored
 
-from .detection_types import Pathlike
+from .types import PathLikeOrStr
 
 __all__ = ["logger", "set_logger_dir", "auto_set_dir", "get_logger_dir"]
 
+ENV_VARS_TRUE: set[str] = {"1", "True", "TRUE", "true", "yes"}
+
+
+@dataclass
+class LoggingRecord:
+    """
+    `LoggingRecord` to pass to the logger in order to distinguish from third party libraries.
+
+    Note:
+        `log_dict` will be added to the log record as a dict.
+
+    Args:
+        msg: The log message.
+        log_dict: Optional dictionary to add to the log record.
+
+    """
+
+    msg: str
+    log_dict: Optional[dict[Union[int, str], Any]] = field(default=None)
+
+    def __post_init__(self) -> None:
+        """log_dict will be added to the log record as a dict."""
+        if self.log_dict is not None:
+            self.log_dict["msg"] = self.msg
+
+    def __str__(self) -> str:
+        """Return the message."""
+        return self.msg
+
 
 class CustomFilter(logging.Filter):
     """A custom filter"""
 
+    filter_third_party_lib = os.environ.get("FILTER_THIRD_PARTY_LIB", "False") in ENV_VARS_TRUE
+
     def filter(self, record: logging.LogRecord) -> bool:
-        log_dict = {}
-        args = record.args
-        str_args = []
-        if args:
-            for arg in args:
-                if isinstance(arg, dict):
-                    log_dict.update(arg)
-                else:
-                    str_args.append(arg)
-        record.args = tuple(str_args)
-        if not hasattr(record, "log_dict"):
-            setattr(record, "log_dict", log_dict)
+        if self.filter_third_party_lib:
+            if not isinstance(record.msg, LoggingRecord):
+                return False
+
         return True
 
 
 class StreamFormatter(logging.Formatter):
     """A custom formatter to produce unified LogRecords"""
 
+    std_out_verbose = os.environ.get("STD_OUT_VERBOSE", "False") in ENV_VARS_TRUE
+
     @no_type_check
     def format(self, record: logging.LogRecord) -> str:
         date = colored("[%(asctime)s @%(filename)s:%(lineno)d]", "green")
         msg = colored("%(message)s", "white")
 
+        if self.std_out_verbose:
+            if isinstance(record.msg, LoggingRecord):
+                verbose_info = f" Additional verbose infos: {repr(record.msg.log_dict)}"
+                msg += verbose_info
+
         if record.levelno == logging.WARNING:
             fmt = f"{date}  {colored('WRN', 'magenta', attrs=['blink'])}  {msg}"
         elif record.levelno == logging.ERROR or record.levelno == logging.CRITICAL:  # pylint: disable=R1714
@@ -86,6 +120,8 @@ class StreamFormatter(logging.Formatter):
 class FileFormatter(logging.Formatter):
     """A custom formatter to produce a loggings in json format"""
 
+    filter_third_party_lib = os.environ.get("FILTER_THIRD_PARTY_LIB", "False") in ENV_VARS_TRUE
+
     @no_type_check
     def format(self, record: logging.LogRecord) -> str:
         message = super().format(record)
@@ -97,13 +133,19 @@ class FileFormatter(logging.Formatter):
             "time": datetime.now().strftime("%m%d-%H%M%S"),
             "message": message,
         }
-        log_dict.update(record.log_dict)
+        if isinstance(record.msg, LoggingRecord):
+            if record.msg.log_dict:
+                log_dict.update(record.msg.log_dict)
+                log_dict.pop("msg")
+        elif not self.filter_third_party_lib:
+            log_dict = {"message": record.msg}
         return json.dumps(log_dict)
 
 
 _LOG_DIR = None
-_CONFIG_DICT: Dict[str, Any] = {
+_CONFIG_DICT: dict[str, Any] = {
     "version": 1,
+    "disable_existing_loggers": False,
     "filters": {"customfilter": {"()": lambda: CustomFilter()}},  # pylint: disable=W0108
     "formatters": {
         "streamformatter": {"()": lambda: StreamFormatter(datefmt="%m%d %H:%M.%S")},
@@ -111,7 +153,11 @@ _CONFIG_DICT: Dict[str, Any] = {
     "handlers": {
         "streamhandler": {"filters": ["customfilter"], "formatter": "streamformatter", "class": "logging.StreamHandler"}
     },
-    "root": {"handlers": ["streamhandler"], "level": "INFO", "propagate": False},
+    "root": {
+        "handlers": ["streamhandler"],
+        "level": os.environ.get("LOG_LEVEL", "INFO"),
+        "propagate": os.environ.get("LOG_PROPAGATE", "False") in ENV_VARS_TRUE,
+    },
 }
 
 
@@ -136,9 +182,8 @@ def _get_time_str() -> str:
     return datetime.now().strftime("%m%d-%H%M%S")
 
 
-def _set_file(path: Pathlike) -> None:
-    if isinstance(path, Path):
-        path = path.as_posix()
+def _set_file(path: PathLikeOrStr) -> None:
+    path = os.fspath(path)
     global _FILE_HANDLER  # pylint: disable=W0603
     if os.path.isfile(path):
         backup_name = path + "." + _get_time_str()
@@ -153,21 +198,20 @@ def _set_file(path: Pathlike) -> None:
     logger.info("Argv: %s ", sys.argv)
 
 
-def set_logger_dir(dir_name: Pathlike, action: Optional[str] = None) -> None:
+def set_logger_dir(dir_name: PathLikeOrStr, action: Optional[str] = None) -> None:
     """
     Set the directory for global logging.
 
-    :param dir_name: log directory
-    :param action: an action of ["k","d","q"] to be performed
-                   when the directory exists. Will ask user by default.
+    Args:
+        dir_name: Log directory.
+        action: An action of ["k", "d", "q"] to be performed when the directory exists. Will ask user by default.
+            "d": Delete the directory. Note that the deletion may fail when the directory is used by tensorboard.
+            "k": Keep the directory. This is useful when you resume from a previous training and want the directory to
+                 look as if the training was not interrupted.
+            Note that this option does not load old models or any other old states for you. It simply does nothing.
 
-                   "d": delete the directory. Note that the deletion may fail when
-                   the directory is used by tensorboard.
-                   "k": keep the directory. This is useful when you resume from a
-                   previous training and want the directory to look as if the
-                   training was not interrupted.
-                   Note that this option does not load old models or any other
-                   old states for you. It simply does nothing.
+    Raises:
+        OSError: If the directory exists and an invalid action is selected.
     """
     if isinstance(dir_name, Path):
         dir_name = dir_name.as_posix()
@@ -178,14 +222,14 @@ def set_logger_dir(dir_name: Pathlike, action: Optional[str] = None) -> None:
         logger.removeHandler(_FILE_HANDLER)
         del _FILE_HANDLER
 
-    def dir_nonempty(directory: str) -> int:
+    def dir_nonempty(directory: PathLikeOrStr) -> int:
         return os.path.isdir(directory) and len([x for x in os.listdir(directory) if x[0] != "."])
 
     if dir_nonempty(dir_name):
         if not action:
             logger.warning("Log directory %s exists! Use 'd' to delete it. ", dir_name)
             logger.warning(
-                "If you're resuming from a previous run, you can choose to keep it." "Press any other key to exit. "
+                "If you're resuming from a previous run, you can choose to keep it. Press any other key to exit. "
             )
         while not action:
             action = input("Select Action: k (keep) / d (delete) / q (quit):").lower().strip()
@@ -218,24 +262,27 @@ def set_logger_dir(dir_name: Pathlike, action: Optional[str] = None) -> None:
 def auto_set_dir(action: Optional[str] = None, name: Optional[str] = None) -> None:
     """
     Will set the log directory to './train_log/{script_name}:{name}'.
-    'script_name' is the name of the main python file currently running.
+    `script_name` is the name of the main python file currently running.
 
-    :param action: an action of ["k","d","q"] to be performed (see also func: set_logger_dir)
-    :param name: Optional suffix of file name.
+    Args:
+        action: An action of ["k", "d", "q"] to be performed (see also `set_logger_dir`).
+        name: Optional suffix of file name.
     """
 
     mod = sys.modules["__main__"]
-    basename = str(os.path.basename(mod.__file__))  # type: ignore
+    basename = str(os.path.basename(mod.__file__))  # type: ignore  # pylint: disable=E1101
     auto_dir_name = os.path.join("train_log", basename[: basename.rfind(".")])
     if name:
         auto_dir_name += "_%s" % name if os.name == "nt" else ":%s" % name  # pylint: disable=C0209
     set_logger_dir(auto_dir_name, action=action)
 
 
-def get_logger_dir() -> Optional[str]:
+def get_logger_dir() -> Optional[PathLikeOrStr]:
     """
-    The logger directory, or None if not set.
-    The directory is used for general logging, tensorboard events, checkpoints, etc.
+    The logger directory, or `None` if not set.
+
+    Returns:
+        The directory used for general logging, tensorboard events, checkpoints, etc.
     """
     return _LOG_DIR
 
@@ -243,11 +290,16 @@ def get_logger_dir() -> Optional[str]:
 @functools.lru_cache(maxsize=None)
 def log_once(message: str, function: str = "info") -> None:
     """
-    Log certain message only once. Call this function more than one times with
-    the same message will result in no-op.
+    Log certain message only once. Calling this function more than once with
+    the same message will result in no operation.
 
-    :param message:  message to log
-    :param function: the name of the logger method. e.g. "info", "warn", "error".
-    :return:
+    Example:
+        ```python
+        log_once("This will only be logged once", "info")
+        ```
+
+    Args:
+        message: Message to log.
+        function: The name of the logger method. For example, "info", "warn", "error".
     """
     getattr(logger, function)(message)
