@@ -16,22 +16,29 @@
 # limitations under the License.
 
 """
-Class AttrDict for maintaining configs and some functions for generating and saving AttrDict instances to .yaml files
+Class `AttrDict` for maintaining configs and some functions for generating and saving `AttrDict` instances to
+`.yaml` files
 """
+from __future__ import annotations
 
 import pprint
-from typing import Any, Dict, List
+from typing import Any
 
 import yaml
 
-from .detection_types import Pathlike
+from .types import PathLikeOrStr
 
 
 # Copyright (c) Tensorpack Contributors
 # Licensed under the Apache License, Version 2.0 (the "License")
 class AttrDict:
     """
-    Class for storing key,values as instance with attributes and values.
+    Class `AttrDict` for maintaining configs and some functions for generating and saving `AttrDict` instances to
+    `.yaml` files.
+
+    Info:
+        This module provides a class for storing key-value pairs as attributes and functions for serializing and
+        deserializing configurations.
     """
 
     _freezed = False
@@ -40,7 +47,17 @@ class AttrDict:
 
     def __getattr__(self, name: str) -> Any:
         """
-        __getattr__
+        Returns the attribute value for `name`. If the attribute does not exist and the instance is not frozen, a new
+        `AttrDict` is created and assigned.
+
+        Args:
+            name: The name of the attribute.
+
+        Returns:
+            The value of the attribute.
+
+        Raises:
+            AttributeError: If the instance is frozen or the attribute name starts with `_`.
         """
         if self._freezed:
             raise AttributeError(name)
@@ -53,29 +70,47 @@ class AttrDict:
 
     def __setattr__(self, name: str, value: Any) -> None:
         """
-        __setattr__
+        Sets the attribute `name` to `value`.
+
+        Args:
+            name: The name of the attribute.
+            value: The value to set.
+
+        Raises:
+            AttributeError: If the instance is frozen and `name` is not `_freezed`.
         """
-        if self._freezed and name not in self.__dict__:
+        if self._freezed and name != "_freezed":
             raise AttributeError(f"Config was freezed! Unknown config: {name}")
         super().__setattr__(name, value)
 
     def __str__(self) -> str:
         """
-        __str__
+        Returns a pretty-printed string representation of the configuration.
+
+        Returns:
+            A string representation of the configuration.
         """
         return pprint.pformat(self.to_dict(), width=100, compact=True)
 
     __repr__ = __str__
 
-    def to_dict(self) -> Dict[str, Any]:
-        """Convert to a nested dict."""
+    def to_dict(self) -> dict[str, Any]:
+        """
+        Convert to a nested dict.
+
+        Returns:
+            A dictionary representation of the configuration.
+        """
         return {
             k: v.to_dict() if isinstance(v, AttrDict) else v for k, v in self.__dict__.items() if not k.startswith("_")
         }
 
-    def from_dict(self, d: Dict[str, Any]) -> None:  # pylint: disable=C0103
+    def from_dict(self, d: dict[str, Any]) -> None:  # pylint: disable=C0103
         """
-        Generate an instance from a dict
+        Generate an instance from a dict.
+
+        Args:
+            d: The dictionary to load values from.
         """
         if isinstance(d, dict):
             self.freeze(False)
@@ -86,9 +121,12 @@ class AttrDict:
                 else:
                     setattr(self, k, v)
 
-    def update_args(self, args: List[str]) -> None:
+    def update_args(self, args: list[str]) -> None:
         """
         Update from command line args.
+
+        Args:
+            args: A list of command line arguments in the form `key1.key2=val`.
         """
         for cfg in args:
             keys, v = cfg.split("=", maxsplit=1)  # pylint: disable=C0103
@@ -105,9 +143,26 @@ class AttrDict:
                 v = eval(v)  # pylint: disable=C0103, W0123
             setattr(dic, key, v)
 
+    def overwrite_config(self, other_config: AttrDict) -> None:
+        """
+        Overwrite the current config with values from another config.
+
+        Args:
+            other_config: The other `AttrDict` instance to copy values from.
+
+        Raises:
+            AttributeError: If the config is frozen.
+        """
+        if self._freezed:
+            raise AttributeError("Config was freezed! Cannot overwrite config.")
+        self.from_dict(other_config.to_dict())
+
     def freeze(self, freezed: bool = True) -> None:
         """
-        :param freezed: freeze the instance, so that no attributes can be added or changed
+        Freeze or unfreeze the instance, so that no attributes can be added or changed.
+
+        Args:
+            freezed: Whether to freeze the instance.
         """
         self._freezed = freezed
         for v in self.__dict__.values():  # pylint: disable=C0103
@@ -122,11 +177,15 @@ class AttrDict:
         raise NotImplementedError()
 
 
-def set_config_by_yaml(path_yaml: Pathlike) -> AttrDict:
+def set_config_by_yaml(path_yaml: PathLikeOrStr) -> AttrDict:
     """
-    Use to initialize the config class for tensorpack faster rcnn
+    Initialize the config class from a YAML file.
 
-    :param path_yaml: The path to the file
+    Args:
+        path_yaml: The path to the YAML file.
+
+    Returns:
+        An `AttrDict` instance initialized from the YAML file.
     """
     config = AttrDict()
     _C = config  # pylint: disable=C0103
@@ -139,11 +198,19 @@ def set_config_by_yaml(path_yaml: Pathlike) -> AttrDict:
     return config
 
 
-def save_config_to_yaml(config: AttrDict, path_yaml: Pathlike) -> None:
+def save_config_to_yaml(config: AttrDict, path_yaml: PathLikeOrStr) -> None:
     """
-    :param config: The configuration instance as an AttrDict
-    :param path_yaml: Save the config class for tensorpack faster rcnn
-    :return: yaml_path: The path to save the file to
+    Save the configuration instance as a YAML file.
+
+    Example:
+        ```python
+        save_config_to_yaml(config, "config.yaml")
+        ```
+
+    Args:
+        config: The configuration instance as an `AttrDict`.
+        path_yaml: The path to save the YAML file to.
+
     """
 
     with open(path_yaml, "w") as file:  # pylint: disable=W1514
@@ -152,12 +219,19 @@ def save_config_to_yaml(config: AttrDict, path_yaml: Pathlike) -> None:
 
 def config_to_cli_str(config: AttrDict, *exclude: str) -> str:
     """
-    Transform an AttrDict to a string that can be passed to a cli. Add optionally keys of the config that should not be
-    added to the string.
+    Transform an `AttrDict` to a string that can be passed to a CLI. Optionally exclude keys from the string.
 
-    :param config: An `AttrDict`
-    :param exclude: keys of the AttrDict
-    :return: A string that can be passed to a cli
+    Example:
+        ```python
+        config_to_cli_str(config, "key1", "key2")
+        ```
+
+    Args:
+        config: An `AttrDict`.
+        *exclude: Keys of the `AttrDict` to exclude.
+
+    Returns:
+        A string that can be passed to a CLI.
     """
 
     config_dict = config.to_dict()
